@@ -1,15 +1,11 @@
-# Copyright (c) 2015-2018, 2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2017 Derek Gustafson <degustaf@gmail.com>
-# Copyright (c) 2019, 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
 """Checker for anything related to the async protocol (PEP 492)."""
 
 import sys
+from typing import TYPE_CHECKING
 
 import astroid
 from astroid import nodes
@@ -18,6 +14,9 @@ from pylint import checkers, interfaces, utils
 from pylint.checkers import utils as checker_utils
 from pylint.checkers.utils import decorated_with
 
+if TYPE_CHECKING:
+    from pylint.lint import PyLinter
+
 
 class AsyncChecker(checkers.BaseChecker):
     __implements__ = interfaces.IAstroidChecker
@@ -43,10 +42,11 @@ class AsyncChecker(checkers.BaseChecker):
         self._ignore_mixin_members = utils.get_global_option(
             self, "ignore-mixin-members"
         )
+        self._mixin_class_rgx = utils.get_global_option(self, "mixin-class-rgx")
         self._async_generators = ["contextlib.asynccontextmanager"]
 
     @checker_utils.check_messages("yield-inside-async-function")
-    def visit_asyncfunctiondef(self, node):
+    def visit_asyncfunctiondef(self, node: nodes.AsyncFunctionDef) -> None:
         for child in node.nodes_of_class(nodes.Yield):
             if child.scope() is node and (
                 sys.version_info[:2] == (3, 5) or isinstance(child, nodes.YieldFrom)
@@ -54,7 +54,7 @@ class AsyncChecker(checkers.BaseChecker):
                 self.add_message("yield-inside-async-function", node=child)
 
     @checker_utils.check_messages("not-async-context-manager")
-    def visit_asyncwith(self, node):
+    def visit_asyncwith(self, node: nodes.AsyncWith) -> None:
         for ctx_mgr, _ in node.items:
             inferred = checker_utils.safe_infer(ctx_mgr)
             if inferred is None or inferred is astroid.Uninferable:
@@ -80,10 +80,11 @@ class AsyncChecker(checkers.BaseChecker):
                         # just skip it.
                         if not checker_utils.has_known_bases(inferred):
                             continue
-                        # Just ignore mixin classes.
-                        if self._ignore_mixin_members:
-                            if inferred.name[-5:].lower() == "mixin":
-                                continue
+                        # Ignore mixin classes if they match the rgx option.
+                        if self._ignore_mixin_members and self._mixin_class_rgx.match(
+                            inferred.name
+                        ):
+                            continue
                 else:
                     continue
             self.add_message(
@@ -91,6 +92,5 @@ class AsyncChecker(checkers.BaseChecker):
             )
 
 
-def register(linter):
-    """required method to auto register this checker"""
+def register(linter: "PyLinter") -> None:
     linter.register_checker(AsyncChecker(linter))
