@@ -1,32 +1,13 @@
-# Copyright (c) 2006, 2009-2010, 2012-2015 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2012, 2014 Google, Inc.
-# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
-# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
-# Copyright (c) 2016 Łukasz Rogalski <rogalski.91@gmail.com>
-# Copyright (c) 2017 ahirnish <ahirnish@gmail.com>
-# Copyright (c) 2018 Lucas Cimon <lucas.cimon@gmail.com>
-# Copyright (c) 2018 Mike Frysinger <vapier@gmail.com>
-# Copyright (c) 2018 Mark Miller <725mrm@gmail.com>
-# Copyright (c) 2018 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
-# Copyright (c) 2018 Jakub Wilk <jwilk@jwilk.net>
-# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2019 Michael Scott Cuthbert <cuthbert@mit.edu>
-# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
-# Copyright (c) 2021 Rebecca Turner <rbt@sent.as>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-# Copyright (c) 2021 yushao2 <36848472+yushao2@users.noreply.github.com>
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
-"""check for signs of poor design"""
+"""Check for signs of poor design."""
 
 import re
+import sys
 from collections import defaultdict
-from typing import FrozenSet, List, Set, cast
+from typing import TYPE_CHECKING, FrozenSet, Iterator, List, Set, cast
 
 import astroid
 from astroid import nodes
@@ -36,6 +17,14 @@ from pylint.checkers import BaseChecker
 from pylint.checkers.utils import check_messages
 from pylint.interfaces import IAstroidChecker
 
+if sys.version_info >= (3, 8):
+    from functools import cached_property
+else:
+    from astroid.decorators import cachedproperty as cached_property
+
+if TYPE_CHECKING:
+    from pylint.lint import PyLinter
+
 MSGS = {  # pylint: disable=consider-using-namedtuple-or-dataclass
     "R0901": (
         "Too many ancestors (%s/%s)",
@@ -176,7 +165,7 @@ STDLIB_CLASSES_IGNORE_ANCESTOR = frozenset(
         "typing.ItemsView",
         "typing.ValuesView",
         "typing.ContextManager",
-        "typing.AsyncContextManger",
+        "typing.AsyncContextManager",
         "typing.Hashable",
         "typing.Sized",
     )
@@ -184,7 +173,7 @@ STDLIB_CLASSES_IGNORE_ANCESTOR = frozenset(
 
 
 def _is_exempt_from_public_methods(node: astroid.ClassDef) -> bool:
-    """Check if a class is exempt from too-few-public-methods"""
+    """Check if a class is exempt from too-few-public-methods."""
 
     # If it's a typing.Namedtuple, typing.TypedDict or an Enum
     for ancestor in node.ancestors():
@@ -216,7 +205,7 @@ def _is_exempt_from_public_methods(node: astroid.ClassDef) -> bool:
 
 
 def _count_boolean_expressions(bool_op):
-    """Counts the number of boolean expressions in BoolOp `bool_op` (recursive)
+    """Counts the number of boolean expressions in BoolOp `bool_op` (recursive).
 
     example: a and (b or c or (d and e)) ==> 5 boolean expressions
     """
@@ -239,9 +228,9 @@ def _count_methods_in_class(node):
     return all_methods
 
 
-def _get_parents(
+def _get_parents_iter(
     node: nodes.ClassDef, ignored_parents: FrozenSet[str]
-) -> Set[nodes.ClassDef]:
+) -> Iterator[nodes.ClassDef]:
     r"""Get parents of ``node``, excluding ancestors of ``ignored_parents``.
 
     If we have the following inheritance diagram:
@@ -263,13 +252,28 @@ def _get_parents(
         parent = to_explore.pop()
         if parent.qname() in ignored_parents:
             continue
-        parents.add(parent)
-        to_explore.extend(parent.ancestors(recurs=False))  # type: ignore
-    return parents
+        if parent not in parents:
+            # This guard might appear to be performing the same function as
+            # adding the resolved parents to a set to eliminate duplicates
+            # (legitimate due to diamond inheritance patterns), but its
+            # additional purpose is to prevent cycles (not normally possible,
+            # but potential due to inference) and thus guarantee termination
+            # of the while-loop
+            yield parent
+            parents.add(parent)
+            to_explore.extend(parent.ancestors(recurs=False))
+
+
+def _get_parents(
+    node: nodes.ClassDef, ignored_parents: FrozenSet[str]
+) -> Set[nodes.ClassDef]:
+    return set(_get_parents_iter(node, ignored_parents))
 
 
 class MisdesignChecker(BaseChecker):
-    """checks for sign of poor/misdesign:
+    """Checker of potential misdesigns.
+
+    Checks for sign of poor/misdesign:
     * number of methods, attributes, local variables...
     * size, complexity of functions, methods
     """
@@ -344,7 +348,7 @@ class MisdesignChecker(BaseChecker):
                 "default": (),
                 "type": "csv",
                 "metavar": "<comma separated list of class names>",
-                "help": "List of qualified class names to ignore when countint class parents (see R0901)",
+                "help": "List of qualified class names to ignore when counting class parents (see R0901)",
             },
         ),
         (
@@ -387,27 +391,39 @@ class MisdesignChecker(BaseChecker):
                 "statement (see R0916).",
             },
         ),
+        (
+            "exclude-too-few-public-methods",
+            {
+                "default": [],
+                "type": "regexp_csv",
+                "metavar": "<pattern>[,<pattern>...]",
+                "help": "List of regular expressions of class ancestor names "
+                "to ignore when counting public methods (see R0903)",
+            },
+        ),
     )
 
     def __init__(self, linter=None):
-        BaseChecker.__init__(self, linter)
-        self.stats = None
+        super().__init__(linter)
         self._returns = None
         self._branches = None
         self._stmts = None
 
     def open(self):
-        """initialize visit variables"""
-        self.stats = self.linter.add_stats()
+        """Initialize visit variables."""
+        self.linter.stats.reset_node_count()
         self._returns = []
         self._branches = defaultdict(int)
         self._stmts = []
+        self._exclude_too_few_public_methods = utils.get_global_option(
+            self, "exclude-too-few-public-methods", default=[]
+        )
 
     def _inc_all_stmts(self, amount):
         for i, _ in enumerate(self._stmts):
             self._stmts[i] += amount
 
-    @astroid.decorators.cachedproperty
+    @cached_property
     def _ignored_argument_names(self):
         return utils.get_global_option(self, "ignored-argument-names", default=None)
 
@@ -417,8 +433,8 @@ class MisdesignChecker(BaseChecker):
         "too-few-public-methods",
         "too-many-public-methods",
     )
-    def visit_classdef(self, node: nodes.ClassDef):
-        """check size of inheritance hierarchy and number of instance attributes"""
+    def visit_classdef(self, node: nodes.ClassDef) -> None:
+        """Check size of inheritance hierarchy and number of instance attributes."""
         parents = _get_parents(
             node, STDLIB_CLASSES_IGNORE_ANCESTOR.union(self.config.ignored_parents)
         )
@@ -438,8 +454,8 @@ class MisdesignChecker(BaseChecker):
             )
 
     @check_messages("too-few-public-methods", "too-many-public-methods")
-    def leave_classdef(self, node):
-        """check number of public methods"""
+    def leave_classdef(self, node: nodes.ClassDef) -> None:
+        """Check number of public methods."""
         my_methods = sum(
             1 for method in node.mymethods() if not method.name.startswith("_")
         )
@@ -458,6 +474,15 @@ class MisdesignChecker(BaseChecker):
                 args=(my_methods, self.config.max_public_methods),
             )
 
+        # Stop here if the class is excluded via configuration.
+        if node.type == "class" and self._exclude_too_few_public_methods:
+            for ancestor in node.ancestors():
+                if any(
+                    pattern.match(ancestor.qname())
+                    for pattern in self._exclude_too_few_public_methods
+                ):
+                    return
+
         # Stop here for exception, metaclass, interface classes and other
         # classes for which we don't need to count the methods.
         if node.type != "class" or _is_exempt_from_public_methods(node):
@@ -482,8 +507,8 @@ class MisdesignChecker(BaseChecker):
         "too-many-statements",
         "keyword-arg-before-vararg",
     )
-    def visit_functiondef(self, node):
-        """check function name, docstring, arguments, redefinition,
+    def visit_functiondef(self, node: nodes.FunctionDef) -> None:
+        """Check function name, docstring, arguments, redefinition,
         variable names, max locals
         """
         # init branch and returns counters
@@ -525,8 +550,8 @@ class MisdesignChecker(BaseChecker):
         "too-many-locals",
         "too-many-statements",
     )
-    def leave_functiondef(self, node):
-        """most of the work is done here on close:
+    def leave_functiondef(self, node: nodes.FunctionDef) -> None:
+        """Most of the work is done here on close:
         checks for max returns, branch, return in __init__
         """
         returns = self._returns.pop()
@@ -554,35 +579,35 @@ class MisdesignChecker(BaseChecker):
 
     leave_asyncfunctiondef = leave_functiondef
 
-    def visit_return(self, _):
-        """count number of returns"""
+    def visit_return(self, _: nodes.Return) -> None:
+        """Count number of returns."""
         if not self._returns:
             return  # return outside function, reported by the base checker
         self._returns[-1] += 1
 
-    def visit_default(self, node):
-        """default visit method -> increments the statements counter if
+    def visit_default(self, node: nodes.NodeNG) -> None:
+        """Default visit method -> increments the statements counter if
         necessary
         """
         if node.is_statement:
             self._inc_all_stmts(1)
 
-    def visit_tryexcept(self, node):
-        """increments the branches counter"""
+    def visit_tryexcept(self, node: nodes.TryExcept) -> None:
+        """Increments the branches counter."""
         branches = len(node.handlers)
         if node.orelse:
             branches += 1
         self._inc_branch(node, branches)
         self._inc_all_stmts(branches)
 
-    def visit_tryfinally(self, node):
-        """increments the branches counter"""
+    def visit_tryfinally(self, node: nodes.TryFinally) -> None:
+        """Increments the branches counter."""
         self._inc_branch(node, 2)
         self._inc_all_stmts(2)
 
     @check_messages("too-many-boolean-expressions")
-    def visit_if(self, node):
-        """increments the branches counter and checks boolean expressions"""
+    def visit_if(self, node: nodes.If) -> None:
+        """Increments the branches counter and checks boolean expressions."""
         self._check_boolean_expressions(node)
         branches = 1
         # don't double count If nodes coming from some 'elif'
@@ -594,9 +619,8 @@ class MisdesignChecker(BaseChecker):
         self._inc_all_stmts(branches)
 
     def _check_boolean_expressions(self, node):
-        """Go through "if" node `node` and counts its boolean expressions
-
-        if the "if" node test is a BoolOp node
+        """Go through "if" node `node` and count its boolean expressions
+        if the 'if' node test is a BoolOp node
         """
         condition = node.test
         if not isinstance(condition, astroid.BoolOp):
@@ -609,8 +633,8 @@ class MisdesignChecker(BaseChecker):
                 args=(nb_bool_expr, self.config.max_bool_expr),
             )
 
-    def visit_while(self, node):
-        """increments the branches counter"""
+    def visit_while(self, node: nodes.While) -> None:
+        """Increments the branches counter."""
         branches = 1
         if node.orelse:
             branches += 1
@@ -619,10 +643,9 @@ class MisdesignChecker(BaseChecker):
     visit_for = visit_while
 
     def _inc_branch(self, node, branchesnum=1):
-        """increments the branches counter"""
+        """Increments the branches counter."""
         self._branches[node.scope()] += branchesnum
 
 
-def register(linter):
-    """required method to auto register this checker"""
+def register(linter: "PyLinter") -> None:
     linter.register_checker(MisdesignChecker(linter))
