@@ -1,47 +1,21 @@
-# Copyright (c) 2006-2011, 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2011-2014 Google, Inc.
-# Copyright (c) 2012 Tim Hatch <tim@timhatch.com>
-# Copyright (c) 2013-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2014 Brett Cannon <brett@python.org>
-# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
-# Copyright (c) 2015 Rene Zhang <rz99@cornell.edu>
-# Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
-# Copyright (c) 2015 Steven Myint <hg@stevenmyint.com>
-# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
-# Copyright (c) 2016 Erik <erik.eriksson@yahoo.com>
-# Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
-# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
-# Copyright (c) 2017 Martin von Gagern <gagern@google.com>
-# Copyright (c) 2018 Lucas Cimon <lucas.cimon@gmail.com>
-# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
-# Copyright (c) 2018 Natalie Serebryakova <natalie.serebryakova@Natalies-MacBook-Pro.local>
-# Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
-# Copyright (c) 2018 Carey Metcalfe <carey@cmetcalfe.ca>
-# Copyright (c) 2018 Mike Frysinger <vapier@gmail.com>
-# Copyright (c) 2018 Alexander Todorov <atodorov@otb.bg>
-# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
-# Copyright (c) 2019, 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2019 Djailla <bastien.vallet@gmail.com>
-# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
-# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2020 Ram Rachum <ram@rachum.com>
-# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
 """Checks for various exception related errors."""
 import builtins
 import inspect
-import typing
+from typing import TYPE_CHECKING, Any, List, Optional
 
 import astroid
-from astroid import nodes
+from astroid import nodes, objects
 
 from pylint import checkers, interfaces
 from pylint.checkers import utils
 
+if TYPE_CHECKING:
+    from pylint.lint import PyLinter
+
 
 def _builtin_exceptions():
     def predicate(obj):
@@ -52,8 +26,8 @@ def _builtin_exceptions():
 
 
 def _annotated_unpack_infer(stmt, context=None):
-    """
-    Recursively generate nodes inferred by the given statement.
+    """Recursively generate nodes inferred by the given statement.
+
     If the inferred value is a list or a tuple, recurse on the elements.
     Returns an iterator which yields tuples in the format
     ('original node', 'inferred node').
@@ -70,12 +44,9 @@ def _annotated_unpack_infer(stmt, context=None):
         yield stmt, inferred
 
 
-def _is_raising(body: typing.List) -> bool:
-    """Return true if the given statement node raise an exception"""
-    for node in body:
-        if isinstance(node, nodes.Raise):
-            return True
-    return False
+def _is_raising(body: List) -> bool:
+    """Return whether the given statement node raises an exception."""
+    return any(isinstance(node, nodes.Raise) for node in body)
 
 
 OVERGENERAL_EXCEPTIONS = ("BaseException", "Exception")
@@ -91,8 +62,8 @@ MSGS = {  # pylint: disable=consider-using-namedtuple-or-dataclass
     "E0702": (
         "Raising %s while only classes or instances are allowed",
         "raising-bad-type",
-        "Used when something which is neither a class, an instance or a "
-        "string is raised (i.e. a `TypeError` will be raised).",
+        "Used when something which is neither a class nor an instance "
+        "is raised (i.e. a `TypeError` will be raised).",
     ),
     "E0703": (
         "Exception context set to something which is not an exception, nor None",
@@ -201,26 +172,26 @@ class BaseVisitor:
         else:
             self.visit_default(node)
 
-    def visit_default(self, node):  # pylint: disable=unused-argument
+    def visit_default(self, _: nodes.NodeNG) -> None:
         """Default implementation for all the nodes."""
 
 
 class ExceptionRaiseRefVisitor(BaseVisitor):
     """Visit references (anything that is not an AST leaf)."""
 
-    def visit_name(self, name):
-        if name.name == "NotImplemented":
+    def visit_name(self, node: nodes.Name) -> None:
+        if node.name == "NotImplemented":
             self._checker.add_message("notimplemented-raised", node=self._node)
 
-    def visit_call(self, call):
-        if isinstance(call.func, nodes.Name):
-            self.visit_name(call.func)
+    def visit_call(self, node: nodes.Call) -> None:
+        if isinstance(node.func, nodes.Name):
+            self.visit_name(node.func)
         if (
-            len(call.args) > 1
-            and isinstance(call.args[0], nodes.Const)
-            and isinstance(call.args[0].value, str)
+            len(node.args) > 1
+            and isinstance(node.args[0], nodes.Const)
+            and isinstance(node.args[0].value, str)
         ):
-            msg = call.args[0].value
+            msg = node.args[0].value
             if "%" in msg or ("{" in msg and "}" in msg):
                 self._checker.add_message("raising-format-tuple", node=self._node)
 
@@ -228,30 +199,27 @@ class ExceptionRaiseRefVisitor(BaseVisitor):
 class ExceptionRaiseLeafVisitor(BaseVisitor):
     """Visitor for handling leaf kinds of a raise value."""
 
-    def visit_const(self, const):
-        if not isinstance(const.value, str):
-            # raising-string will be emitted from python3 porting checker.
-            self._checker.add_message(
-                "raising-bad-type", node=self._node, args=const.value.__class__.__name__
-            )
+    def visit_const(self, node: nodes.Const) -> None:
+        self._checker.add_message(
+            "raising-bad-type", node=self._node, args=node.value.__class__.__name__
+        )
 
-    def visit_instance(self, instance):
-        # pylint: disable=protected-access
+    def visit_instance(self, instance: objects.ExceptionInstance) -> None:
         cls = instance._proxied
         self.visit_classdef(cls)
 
     # Exception instances have a particular class type
     visit_exceptioninstance = visit_instance
 
-    def visit_classdef(self, cls):
-        if not utils.inherit_from_std_ex(cls) and utils.has_known_bases(cls):
-            if cls.newstyle:
+    def visit_classdef(self, node: nodes.ClassDef) -> None:
+        if not utils.inherit_from_std_ex(node) and utils.has_known_bases(node):
+            if node.newstyle:
                 self._checker.add_message("raising-non-exception", node=self._node)
 
-    def visit_tuple(self, _):
+    def visit_tuple(self, _: nodes.Tuple) -> None:
         self._checker.add_message("raising-bad-type", node=self._node, args="tuple")
 
-    def visit_default(self, node):
+    def visit_default(self, node: nodes.NodeNG) -> None:
         name = getattr(node, "name", node.__class__.__name__)
         self._checker.add_message("raising-bad-type", node=self._node, args=name)
 
@@ -271,7 +239,7 @@ class ExceptionsChecker(checkers.BaseChecker):
                 "default": OVERGENERAL_EXCEPTIONS,
                 "type": "csv",
                 "metavar": "<comma-separated class names>",
-                "help": "Exceptions that will emit a warning "
+                "help": "Exceptions that will emit a warning "  # pylint: disable=consider-using-f-string
                 'when being caught. Defaults to "%s".'
                 % (", ".join(OVERGENERAL_EXCEPTIONS),),
             },
@@ -291,7 +259,7 @@ class ExceptionsChecker(checkers.BaseChecker):
         "raising-format-tuple",
         "raise-missing-from",
     )
-    def visit_raise(self, node):
+    def visit_raise(self, node: nodes.Raise) -> None:
         if node.exc is None:
             self._check_misplaced_bare_raise(node)
             return
@@ -304,13 +272,10 @@ class ExceptionsChecker(checkers.BaseChecker):
         expr = node.exc
         ExceptionRaiseRefVisitor(self, node).visit(expr)
 
-        try:
-            inferred_value = expr.inferred()[-1]
-        except astroid.InferenceError:
-            pass
-        else:
-            if inferred_value:
-                ExceptionRaiseLeafVisitor(self, node).visit(inferred_value)
+        inferred = utils.safe_infer(expr)
+        if inferred is None or inferred is astroid.Uninferable:
+            return
+        ExceptionRaiseLeafVisitor(self, node).visit(inferred)
 
     def _check_misplaced_bare_raise(self, node):
         # Filter out if it's present in __exit__.
@@ -430,8 +395,8 @@ class ExceptionsChecker(checkers.BaseChecker):
     def _check_try_except_raise(self, node):
         def gather_exceptions_from_handler(
             handler,
-        ) -> typing.Optional[typing.List[nodes.NodeNG]]:
-            exceptions: typing.List[nodes.NodeNG] = []
+        ) -> Optional[List[nodes.NodeNG]]:
+            exceptions: List[nodes.NodeNG] = []
             if handler.type:
                 exceptions_in_handler = utils.safe_infer(handler.type)
                 if isinstance(exceptions_in_handler, nodes.Tuple):
@@ -451,7 +416,7 @@ class ExceptionsChecker(checkers.BaseChecker):
 
         bare_raise = False
         handler_having_bare_raise = None
-        excs_in_bare_handler = []
+        exceptions_in_bare_handler = []
         for handler in node.handlers:
             if bare_raise:
                 # check that subsequent handler is not parent of handler which had bare raise.
@@ -461,14 +426,14 @@ class ExceptionsChecker(checkers.BaseChecker):
                 excs_in_current_handler = gather_exceptions_from_handler(handler)
                 if not excs_in_current_handler:
                     break
-                if excs_in_bare_handler is None:
+                if exceptions_in_bare_handler is None:
                     # It can be `None` when the inference failed
                     break
                 for exc_in_current_handler in excs_in_current_handler:
                     inferred_current = utils.safe_infer(exc_in_current_handler)
                     if any(
                         utils.is_subclass_of(utils.safe_infer(e), inferred_current)
-                        for e in excs_in_bare_handler
+                        for e in exceptions_in_bare_handler
                     ):
                         bare_raise = False
                         break
@@ -479,29 +444,23 @@ class ExceptionsChecker(checkers.BaseChecker):
                 if handler.body[0].exc is None:
                     bare_raise = True
                     handler_having_bare_raise = handler
-                    excs_in_bare_handler = gather_exceptions_from_handler(handler)
+                    exceptions_in_bare_handler = gather_exceptions_from_handler(handler)
         else:
             if bare_raise:
                 self.add_message("try-except-raise", node=handler_having_bare_raise)
 
     @utils.check_messages("wrong-exception-operation")
-    def visit_binop(self, node):
+    def visit_binop(self, node: nodes.BinOp) -> None:
         if isinstance(node.parent, nodes.ExceptHandler):
             # except (V | A)
-            suggestion = "Did you mean '({}, {})' instead?".format(
-                node.left.as_string(),
-                node.right.as_string(),
-            )
+            suggestion = f"Did you mean '({node.left.as_string()}, {node.right.as_string()})' instead?"
             self.add_message("wrong-exception-operation", node=node, args=(suggestion,))
 
     @utils.check_messages("wrong-exception-operation")
-    def visit_compare(self, node):
+    def visit_compare(self, node: nodes.Compare) -> None:
         if isinstance(node.parent, nodes.ExceptHandler):
             # except (V < A)
-            suggestion = "Did you mean '({}, {})' instead?".format(
-                node.left.as_string(),
-                ", ".join(operand.as_string() for _, operand in node.ops),
-            )
+            suggestion = f"Did you mean '({node.left.as_string()}, {', '.join(operand.as_string() for _, operand in node.ops)})' instead?"
             self.add_message("wrong-exception-operation", node=node, args=(suggestion,))
 
     @utils.check_messages(
@@ -513,10 +472,10 @@ class ExceptionsChecker(checkers.BaseChecker):
         "catching-non-exception",
         "duplicate-except",
     )
-    def visit_tryexcept(self, node):
-        """check for empty except"""
+    def visit_tryexcept(self, node: nodes.TryExcept) -> None:
+        """Check for empty except."""
         self._check_try_except_raise(node)
-        exceptions_classes = []
+        exceptions_classes: List[Any] = []
         nb_handlers = len(node.handlers)
         for index, handler in enumerate(node.handlers):
             if handler.type is None:
@@ -535,56 +494,51 @@ class ExceptionsChecker(checkers.BaseChecker):
                 )
             else:
                 try:
-                    excs = list(_annotated_unpack_infer(handler.type))
+                    exceptions = list(_annotated_unpack_infer(handler.type))
                 except astroid.InferenceError:
                     continue
 
-                for part, exc in excs:
-                    if exc is astroid.Uninferable:
+                for part, exception in exceptions:
+                    if exception is astroid.Uninferable:
                         continue
-                    if isinstance(exc, astroid.Instance) and utils.inherit_from_std_ex(
-                        exc
-                    ):
-                        # pylint: disable=protected-access
-                        exc = exc._proxied
+                    if isinstance(
+                        exception, astroid.Instance
+                    ) and utils.inherit_from_std_ex(exception):
+                        exception = exception._proxied
 
-                    self._check_catching_non_exception(handler, exc, part)
+                    self._check_catching_non_exception(handler, exception, part)
 
-                    if not isinstance(exc, nodes.ClassDef):
+                    if not isinstance(exception, nodes.ClassDef):
                         continue
 
                     exc_ancestors = [
                         anc
-                        for anc in exc.ancestors()
+                        for anc in exception.ancestors()
                         if isinstance(anc, nodes.ClassDef)
                     ]
 
                     for previous_exc in exceptions_classes:
                         if previous_exc in exc_ancestors:
-                            msg = "{} is an ancestor class of {}".format(
-                                previous_exc.name,
-                                exc.name,
-                            )
+                            msg = f"{previous_exc.name} is an ancestor class of {exception.name}"
                             self.add_message(
                                 "bad-except-order", node=handler.type, args=msg
                             )
                     if (
-                        exc.name in self.config.overgeneral_exceptions
-                        and exc.root().name == utils.EXCEPTIONS_MODULE
+                        exception.name in self.config.overgeneral_exceptions
+                        and exception.root().name == utils.EXCEPTIONS_MODULE
                         and not _is_raising(handler.body)
                     ):
                         self.add_message(
-                            "broad-except", args=exc.name, node=handler.type
+                            "broad-except", args=exception.name, node=handler.type
                         )
 
-                    if exc in exceptions_classes:
+                    if exception in exceptions_classes:
                         self.add_message(
-                            "duplicate-except", args=exc.name, node=handler.type
+                            "duplicate-except", args=exception.name, node=handler.type
                         )
 
-                exceptions_classes += [exc for _, exc in excs]
+                exceptions_classes += [exc for _, exc in exceptions]
 
 
-def register(linter):
-    """required method to auto register this checker"""
+def register(linter: "PyLinter") -> None:
     linter.register_checker(ExceptionsChecker(linter))
