@@ -1,58 +1,13 @@
-# Copyright (c) 2006-2015 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2012-2014 Google, Inc.
-# Copyright (c) 2013 buck@yelp.com <buck@yelp.com>
-# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2014 Brett Cannon <brett@python.org>
-# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
-# Copyright (c) 2015-2016 Moises Lopez <moylop260@vauxoo.com>
-# Copyright (c) 2015 Dmitry Pribysh <dmand@yandex.ru>
-# Copyright (c) 2015 Cezar <celnazli@bitdefender.com>
-# Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
-# Copyright (c) 2015 Noam Yorav-Raphael <noamraph@gmail.com>
-# Copyright (c) 2015 James Morgensen <james.morgensen@gmail.com>
-# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
-# Copyright (c) 2016, 2021 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2016 Jared Garst <cultofjared@gmail.com>
-# Copyright (c) 2016 Maik Röder <maikroeder@gmail.com>
-# Copyright (c) 2016 Glenn Matthews <glenn@e-dad.net>
-# Copyright (c) 2017, 2020 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2017 Michka Popoff <michkapopoff@gmail.com>
-# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
-# Copyright (c) 2017 Erik Wright <erik.wright@shopify.com>
-# Copyright (c) 2018 Lucas Cimon <lucas.cimon@gmail.com>
-# Copyright (c) 2018 Hornwitser <github@hornwitser.no>
-# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
-# Copyright (c) 2018 Natalie Serebryakova <natalie.serebryakova@Natalies-MacBook-Pro.local>
-# Copyright (c) 2018 Mike Frysinger <vapier@gmail.com>
-# Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
-# Copyright (c) 2018 Marianna Polatoglou <mpolatoglou@bloomberg.net>
-# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2019, 2021 Nick Drozd <nicholasdrozd@gmail.com>
-# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
-# Copyright (c) 2019 Nick Smith <clickthisnick@users.noreply.github.com>
-# Copyright (c) 2019 Paul Renvoisé <renvoisepaul@gmail.com>
-# Copyright (c) 2020 Peter Kolbus <peter.kolbus@gmail.com>
-# Copyright (c) 2020 Damien Baty <damien.baty@polyconseil.fr>
-# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
-# Copyright (c) 2021 Tushar Sadhwani <tushar.sadhwani000@gmail.com>
-# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-# Copyright (c) 2021 Will Shanks <wsha@posteo.net>
-# Copyright (c) 2021 Matus Valo <matusvalo@users.noreply.github.com>
-# Copyright (c) 2021 Yu Shao, Pang <36848472+yushao2@users.noreply.github.com>
-# Copyright (c) 2021 Andrew Howe <howeaj@users.noreply.github.com>
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
-"""imports checkers for Python code"""
+"""Imports checkers for Python code."""
 
 import collections
 import copy
 import os
-import sys
-from distutils import sysconfig
-from typing import Any, Dict, List, Optional, Set, Tuple, Union
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Set, Tuple, Union
 
 import astroid
 from astroid import nodes
@@ -69,13 +24,15 @@ from pylint.checkers.utils import (
 from pylint.exceptions import EmptyReportError
 from pylint.graph import DotBackend, get_cycles
 from pylint.interfaces import IAstroidChecker
-from pylint.lint import PyLinter
 from pylint.reporters.ureports.nodes import Paragraph, Section, VerbatimText
 from pylint.utils import IsortDriver, get_global_option
 
+if TYPE_CHECKING:
+    from pylint.lint import PyLinter
+
 
 def _qualified_names(modname):
-    """Split the names of the given module into subparts
+    """Split the names of the given module into subparts.
 
     For example,
         _qualified_names('pylint.checkers.ImportsChecker')
@@ -87,7 +44,7 @@ def _qualified_names(modname):
 
 
 def _get_first_import(node, context, name, base, level, alias):
-    """return the node where [base.]<name> is imported or None if not found"""
+    """Return the node where [base.]<name> is imported or None if not found."""
     fullname = f"{base}.{name}" if base else name
 
     first = None
@@ -138,7 +95,7 @@ def _ignore_import_failure(node, modname, ignored_modules):
 
 
 def _make_tree_defs(mod_files_list):
-    """get a list of 2-uple (module, list_of_files_which_import_this_module),
+    """Get a list of 2-uple (module, list_of_files_which_import_this_module),
     it will return a dictionary to represent this as a tree
     """
     tree_defs = {}
@@ -151,7 +108,7 @@ def _make_tree_defs(mod_files_list):
 
 
 def _repr_tree_defs(data, indent_str=None):
-    """return a string which represents imports as a tree"""
+    """Return a string which represents imports as a tree."""
     lines = []
     nodes_items = data.items()
     for i, (mod, (sub, files)) in enumerate(sorted(nodes_items, key=lambda x: x[0])):
@@ -160,7 +117,7 @@ def _repr_tree_defs(data, indent_str=None):
             lines.append(f"{mod} {files}")
             sub_indent_str = "  "
         else:
-            lines.append(fr"{indent_str}\-{mod} {files}")
+            lines.append(rf"{indent_str}\-{mod} {files}")
             if i == len(nodes_items) - 1:
                 sub_indent_str = f"{indent_str}  "
             else:
@@ -171,7 +128,7 @@ def _repr_tree_defs(data, indent_str=None):
 
 
 def _dependencies_graph(filename: str, dep_info: Dict[str, Set[str]]) -> str:
-    """write dependencies as a dot (graphviz) file"""
+    """Write dependencies as a dot (graphviz) file."""
     done = {}
     printer = DotBackend(os.path.splitext(os.path.basename(filename))[0], rankdir="LR")
     printer.emit('URL="." node[shape="box"]')
@@ -192,7 +149,7 @@ def _dependencies_graph(filename: str, dep_info: Dict[str, Set[str]]) -> str:
 def _make_graph(
     filename: str, dep_info: Dict[str, Set[str]], sect: Section, gtype: str
 ):
-    """generate a dependencies graph and add some information about it in the
+    """Generate a dependencies graph and add some information about it in the
     report's section
     """
     outputfile = _dependencies_graph(filename, dep_info)
@@ -300,7 +257,9 @@ DEFAULT_PREFERRED_MODULES = ()
 
 
 class ImportsChecker(DeprecatedMixin, BaseChecker):
-    """checks for
+    """BaseChecker for import statements.
+
+    Checks for
     * external modules dependencies
     * relative / wildcard imports
     * cyclic imports
@@ -425,9 +384,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
         ),
     )
 
-    def __init__(
-        self, linter: Optional[PyLinter] = None
-    ):  # pylint: disable=super-init-not-called # See https://github.com/PyCQA/pylint/issues/4941
+    def __init__(self, linter: Optional["PyLinter"] = None) -> None:
         BaseChecker.__init__(self, linter)
         self.import_graph: collections.defaultdict = collections.defaultdict(set)
         self._imports_stack: List[Tuple[Any, Any]] = []
@@ -441,35 +398,8 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
             ("RP0402", "Modules dependencies graph", self._report_dependencies_graph),
         )
 
-        self._site_packages = self._compute_site_packages()
-
-    @staticmethod
-    def _compute_site_packages():
-        def _normalized_path(path):
-            return os.path.normcase(os.path.abspath(path))
-
-        paths = set()
-        real_prefix = getattr(sys, "real_prefix", None)
-        for prefix in filter(None, (real_prefix, sys.prefix)):
-            path = sysconfig.get_python_lib(prefix=prefix)
-            path = _normalized_path(path)
-            paths.add(path)
-
-        # Handle Debian's derivatives /usr/local.
-        if os.path.isfile("/etc/debian_version"):
-            for prefix in filter(None, (real_prefix, sys.prefix)):
-                libpython = os.path.join(
-                    prefix,
-                    "local",
-                    "lib",
-                    "python" + sysconfig.get_python_version(),
-                    "dist-packages",
-                )
-                paths.add(libpython)
-        return paths
-
     def open(self):
-        """called before visiting project (i.e set of modules)"""
+        """Called before visiting project (i.e set of modules)."""
         self.linter.stats.dependencies = {}
         self.linter.stats = self.linter.stats
         self.import_graph = collections.defaultdict(set)
@@ -491,7 +421,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
         return filtered_graph
 
     def close(self):
-        """called before visiting project (i.e set of modules)"""
+        """Called before visiting project (i.e set of modules)."""
         if self.linter.is_message_enabled("cyclic-import"):
             graph = self._import_graph_without_ignored_edges()
             vertices = list(graph)
@@ -504,7 +434,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
 
     @check_messages(*MSGS)
     def visit_import(self, node: nodes.Import) -> None:
-        """triggered when an import statement is seen"""
+        """Triggered when an import statement is seen."""
         self._check_reimport(node)
         self._check_import_as_rename(node)
         self._check_toplevel(node)
@@ -530,7 +460,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
 
     @check_messages(*MSGS)
     def visit_importfrom(self, node: nodes.ImportFrom) -> None:
-        """triggered when a from statement is seen"""
+        """Triggered when a from statement is seen."""
         basename = node.modname
         imported_module = self._get_imported_module(node, basename)
 
@@ -566,10 +496,6 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
         met_from: Set[str] = set()  # set for 'from x import y' style
         current_package = None
         for import_node, import_name in std_imports + ext_imports + loc_imports:
-            if not self.linter.is_message_enabled(
-                "ungrouped-imports", import_node.fromlineno
-            ):
-                continue
             met = met_from if isinstance(import_node, nodes.ImportFrom) else met_import
             package, _, _ = import_name.partition(".")
             if (
@@ -580,6 +506,10 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
             ):
                 self.add_message("ungrouped-imports", node=import_node, args=package)
             current_package = package
+            if not self.linter.is_message_enabled(
+                "ungrouped-imports", import_node.fromlineno
+            ):
+                continue
             met.add(package)
 
         self._imports_stack = []
@@ -673,7 +603,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
                 self.add_message("reimported", node=node, args=(name, node.fromlineno))
 
     def _check_position(self, node):
-        """Check `node` import or importfrom node position is correct
+        """Check `node` import or importfrom node position is correct.
 
         Send a message  if `node` comes before another instruction
         """
@@ -683,7 +613,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
             self.add_message("wrong-import-position", node=node, args=node.as_string())
 
     def _record_import(self, node, importedmodnode):
-        """Record the package `node` imports from"""
+        """Record the package `node` imports from."""
         if isinstance(node, nodes.ImportFrom):
             importedname = node.modname
         else:
@@ -709,7 +639,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
         return any(astroid.are_exclusive(import_node, node) for import_node in imports)
 
     def _check_imports_order(self, _module_node):
-        """Checks imports of module `node` are grouped by category
+        """Checks imports of module `node` are grouped by category.
 
         Imports must follow this order: standard, 3rd party, local
         """
@@ -831,7 +761,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
     def _add_imported_module(
         self, node: Union[nodes.Import, nodes.ImportFrom], importedmodname: str
     ) -> None:
-        """notify an imported module, used to analyze dependencies"""
+        """Notify an imported module, used to analyze dependencies."""
         module_file = node.root().file
         context_name = node.root().name
         base = os.path.splitext(os.path.basename(module_file))[0]
@@ -858,7 +788,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
                 self._module_pkg[context_name] = context_name.rsplit(".", 1)[0]
 
             # handle dependencies
-            dependencies_stat: Dict[str, Union[Set]] = self.linter.stats.dependencies
+            dependencies_stat: Dict[str, Set[str]] = self.linter.stats.dependencies
             importedmodnames = dependencies_stat.setdefault(importedmodname, set())
             if context_name not in importedmodnames:
                 importedmodnames.add(context_name)
@@ -872,7 +802,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
                 self._excluded_edges[context_name].add(importedmodname)
 
     def _check_preferred_module(self, node, mod_path):
-        """check if the module has a preferred replacement"""
+        """Check if the module has a preferred replacement."""
         if mod_path in self.preferred_modules:
             self.add_message(
                 "preferred-module",
@@ -904,11 +834,11 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
                 )
 
     def _check_reimport(self, node, basename=None, level=None):
-        """check if the import is necessary (i.e. not already done)"""
+        """Check if the import is necessary (i.e. not already done)."""
         if not self.linter.is_message_enabled("reimported"):
             return
 
-        frame = node.frame()
+        frame = node.frame(future=True)
         root = node.root()
         contexts = [(frame, level)]
         if root is not frame:
@@ -925,7 +855,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
                     )
 
     def _report_external_dependencies(self, sect, _, _dummy):
-        """return a verbatim layout for displaying dependencies"""
+        """Return a verbatim layout for displaying dependencies."""
         dep_info = _make_tree_defs(self._external_dependencies_info().items())
         if not dep_info:
             raise EmptyReportError()
@@ -933,7 +863,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
         sect.append(VerbatimText(tree_str))
 
     def _report_dependencies_graph(self, sect, _, _dummy):
-        """write dependencies as a dot (graphviz) file"""
+        """Write dependencies as a dot (graphviz) file."""
         dep_info = self.linter.stats.dependencies
         if not dep_info or not (
             self.config.import_graph
@@ -952,7 +882,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
             _make_graph(filename, self._internal_dependencies_info(), sect, "internal ")
 
     def _filter_dependencies_graph(self, internal):
-        """build the internal or the external dependency graph"""
+        """Build the internal or the external dependency graph."""
         graph = collections.defaultdict(set)
         for importee, importers in self.linter.stats.dependencies.items():
             for importer in importers:
@@ -964,14 +894,14 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
 
     @astroid.decorators.cached
     def _external_dependencies_info(self):
-        """return cached external dependencies information or build and
+        """Return cached external dependencies information or build and
         cache them
         """
         return self._filter_dependencies_graph(internal=False)
 
     @astroid.decorators.cached
     def _internal_dependencies_info(self):
-        """return cached internal dependencies information or build and
+        """Return cached internal dependencies information or build and
         cache them
         """
         return self._filter_dependencies_graph(internal=True)
@@ -1018,6 +948,5 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
             )
 
 
-def register(linter):
-    """required method to auto register this checker"""
+def register(linter: "PyLinter") -> None:
     linter.register_checker(ImportsChecker(linter))
