@@ -1,29 +1,10 @@
-# Copyright (c) 2009-2011, 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2009, 2012, 2014 Google, Inc.
-# Copyright (c) 2012 Mike Bryant <leachim@leachim.info>
-# Copyright (c) 2014 Brett Cannon <brett@python.org>
-# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
-# Copyright (c) 2015-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
-# Copyright (c) 2016, 2019-2020 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2016 Chris Murray <chris@chrismurray.scot>
-# Copyright (c) 2017 guillaume2 <guillaume.peillex@gmail.col>
-# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
-# Copyright (c) 2018 Alan Chan <achan961117@gmail.com>
-# Copyright (c) 2018 Yury Gribov <tetra2005@gmail.com>
-# Copyright (c) 2018 Mike Frysinger <vapier@gmail.com>
-# Copyright (c) 2018 Mariatta Wijaya <mariatta@python.org>
-# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2019 Djailla <bastien.vallet@gmail.com>
-# Copyright (c) 2019 Svet <svet@hyperscience.com>
-# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
-"""checker for use of Python logging
-"""
+"""Checker for use of Python logging."""
 import string
+from typing import TYPE_CHECKING, Set
 
 import astroid
 from astroid import nodes
@@ -32,6 +13,9 @@ from pylint import checkers, interfaces
 from pylint.checkers import utils
 from pylint.checkers.utils import check_messages, infer_all
 
+if TYPE_CHECKING:
+    from pylint.lint import PyLinter
+
 MSGS = {  # pylint: disable=consider-using-namedtuple-or-dataclass
     "W1201": (
         "Use %s formatting in logging functions",
@@ -160,12 +144,12 @@ class LoggingChecker(checkers.BaseChecker):
         ),
     )
 
-    def visit_module(self, node):  # pylint: disable=unused-argument
+    def visit_module(self, _: nodes.Module) -> None:
         """Clears any state left in this checker from last module checked."""
         # The code being checked can just as easily "import logging as foo",
         # so it is necessary to process the imports and store in this field
         # what name the logging module is actually given.
-        self._logging_names = set()
+        self._logging_names: Set[str] = set()
         logging_mods = self.config.logging_modules
 
         self._format_style = self.config.logging_format_style
@@ -177,7 +161,7 @@ class LoggingChecker(checkers.BaseChecker):
             if len(parts) > 1:
                 self._from_imports[parts[0]] = parts[1]
 
-    def visit_importfrom(self, node):
+    def visit_importfrom(self, node: nodes.ImportFrom) -> None:
         """Checks to see if a module uses a non-Python logging module."""
         try:
             logging_name = self._from_imports[node.modname]
@@ -187,14 +171,14 @@ class LoggingChecker(checkers.BaseChecker):
         except KeyError:
             pass
 
-    def visit_import(self, node):
+    def visit_import(self, node: nodes.Import) -> None:
         """Checks to see if this module uses Python's built-in logging."""
         for module, as_name in node.names:
             if module in self._logging_modules:
                 self._logging_names.add(as_name or module)
 
     @check_messages(*MSGS)
-    def visit_call(self, node):
+    def visit_call(self, node: nodes.Call) -> None:
         """Checks calls to logging methods."""
 
         def is_logging_name():
@@ -289,23 +273,18 @@ class LoggingChecker(checkers.BaseChecker):
 
     @staticmethod
     def _is_operand_literal_str(operand):
-        """
-        Return True if the operand in argument is a literal string
-        """
+        """Return True if the operand in argument is a literal string."""
         return isinstance(operand, nodes.Const) and operand.name == "str"
 
-    def _check_call_func(self, node):
-        """Checks that function call is not format_string.format().
-
-        Args:
-          node (astroid.node_classes.Call):
-            Call AST node to be checked.
-        """
+    def _check_call_func(self, node: nodes.Call):
+        """Checks that function call is not format_string.format()."""
         func = utils.safe_infer(node.func)
         types = ("str", "unicode")
         methods = ("format",)
-        if is_method_call(func, types, methods) and not is_complex_format_str(
-            func.bound
+        if (
+            isinstance(func, astroid.BoundMethod)
+            and is_method_call(func, types, methods)
+            and not is_complex_format_str(func.bound)
         ):
             self.add_message(
                 "logging-format-interpolation",
@@ -370,14 +349,8 @@ class LoggingChecker(checkers.BaseChecker):
             self.add_message("logging-too-few-args", node=node)
 
 
-def is_complex_format_str(node):
-    """Checks if node represents a string with complex formatting specs.
-
-    Args:
-        node (nodes.NodeNG): AST node to check
-    Returns:
-        bool: True if inferred string uses complex formatting, False otherwise
-    """
+def is_complex_format_str(node: nodes.NodeNG) -> bool:
+    """Return whether the node represents a string with complex formatting specs."""
     inferred = utils.safe_infer(node)
     if inferred is None or not (
         isinstance(inferred, nodes.Const) and isinstance(inferred.value, str)
@@ -388,10 +361,7 @@ def is_complex_format_str(node):
     except ValueError:
         # This format string is invalid
         return False
-    for _, _, format_spec, _ in parsed:
-        if format_spec:
-            return True
-    return False
+    return any(format_spec for (_, _, format_spec, _) in parsed)
 
 
 def _count_supplied_tokens(args):
@@ -410,6 +380,5 @@ def _count_supplied_tokens(args):
     return sum(1 for arg in args if not isinstance(arg, nodes.Keyword))
 
 
-def register(linter):
-    """Required method to auto-register this checker."""
+def register(linter: "PyLinter") -> None:
     linter.register_checker(LoggingChecker(linter))
