@@ -1,35 +1,12 @@
-# Copyright (c) 2006, 2009-2013 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2012-2014 Google, Inc.
-# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2014 Brett Cannon <brett@python.org>
-# Copyright (c) 2014 Alexandru Coman <fcoman@bitdefender.com>
-# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
-# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
-# Copyright (c) 2016 Łukasz Rogalski <rogalski.91@gmail.com>
-# Copyright (c) 2016 glegoux <gilles.legoux@gmail.com>
-# Copyright (c) 2017-2020 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2017 Mikhail Fesenko <proggga@gmail.com>
-# Copyright (c) 2018 Rogalski, Lukasz <lukasz.rogalski@intel.com>
-# Copyright (c) 2018 Lucas Cimon <lucas.cimon@gmail.com>
-# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
-# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2020 wtracy <afishionado@gmail.com>
-# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
-# Copyright (c) 2020 Benny <benny.mueller91@gmail.com>
-# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
-# Copyright (c) 2021 Nick Drozd <nicholasdrozd@gmail.com>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-# Copyright (c) 2021 Konstantina Saketou <56515303+ksaketou@users.noreply.github.com>
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
-
-"""Check source code is ascii only or has an encoding declaration (PEP 263)"""
+"""Check source code is ascii only or has an encoding declaration (PEP 263)."""
 
 import re
 import tokenize
-from typing import List, Optional
+from typing import TYPE_CHECKING, List, Optional
 
 from astroid import nodes
 
@@ -38,6 +15,9 @@ from pylint.interfaces import IRawChecker, ITokenChecker
 from pylint.typing import ManagedMessage
 from pylint.utils.pragma_parser import OPTION_PO, PragmaParserError, parse_pragma
 
+if TYPE_CHECKING:
+    from pylint.lint import PyLinter
+
 
 class ByIdManagedMessagesChecker(BaseChecker):
 
@@ -73,7 +53,9 @@ class ByIdManagedMessagesChecker(BaseChecker):
 
 class EncodingChecker(BaseChecker):
 
-    """checks for:
+    """BaseChecker for encoding issues.
+
+    Checks for:
     * warning notes in the code like FIXME, XXX
     * encoding issues.
     """
@@ -118,9 +100,9 @@ class EncodingChecker(BaseChecker):
 
         notes = "|".join(re.escape(note) for note in self.config.notes)
         if self.config.notes_rgx:
-            regex_string = fr"#\s*({notes}|{self.config.notes_rgx})\b"
+            regex_string = rf"#\s*({notes}|{self.config.notes_rgx})(?=(:|\s|\Z))"
         else:
-            regex_string = fr"#\s*({notes})\b"
+            regex_string = rf"#\s*({notes})(?=(:|\s|\Z))"
 
         self._fixme_pattern = re.compile(regex_string, re.I)
 
@@ -142,7 +124,7 @@ class EncodingChecker(BaseChecker):
         return None
 
     def process_module(self, node: nodes.Module) -> None:
-        """inspect the source file to find encoding problem"""
+        """Inspect the source file to find encoding problem."""
         encoding = node.file_encoding if node.file_encoding else "ascii"
 
         with node.stream() as stream:
@@ -150,7 +132,7 @@ class EncodingChecker(BaseChecker):
                 self._check_encoding(lineno + 1, line, encoding)
 
     def process_tokens(self, tokens):
-        """inspect the source to find fixme problems"""
+        """Inspect the source to find fixme problems."""
         if not self.config.notes:
             return
         comments = (
@@ -195,7 +177,6 @@ class EncodingChecker(BaseChecker):
                 )
 
 
-def register(linter):
-    """required method to auto register this checker"""
+def register(linter: "PyLinter") -> None:
     linter.register_checker(EncodingChecker(linter))
     linter.register_checker(ByIdManagedMessagesChecker(linter))
