@@ -1,6 +1,6 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
-
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
 import astroid
 from astroid import nodes
@@ -10,7 +10,7 @@ from pylint.checkers import utils
 
 
 class NotChecker(checkers.BaseChecker):
-    """checks for too many not in comparison expressions
+    """Checks for too many not in comparison expressions.
 
     - "not not" should trigger a warning
     - "not" followed by a comparison should trigger a warning
@@ -42,7 +42,7 @@ class NotChecker(checkers.BaseChecker):
     skipped_classnames = [f"builtins.{qname}" for qname in ("set", "frozenset")]
 
     @utils.check_messages("unneeded-not")
-    def visit_unaryop(self, node):
+    def visit_unaryop(self, node: nodes.UnaryOp) -> None:
         if node.op != "not":
             return
         operand = node.operand
@@ -62,7 +62,7 @@ class NotChecker(checkers.BaseChecker):
             if operator not in self.reverse_op:
                 return
             # Ignore __ne__ as function of __eq__
-            frame = node.frame()
+            frame = node.frame(future=True)
             if frame.name == "__ne__" and operator == "==":
                 return
             for _type in (utils.node_type(left), utils.node_type(right)):
@@ -75,10 +75,8 @@ class NotChecker(checkers.BaseChecker):
                     and _type.qname() in self.skipped_classnames
                 ):
                     return
-            suggestion = "{} {} {}".format(
-                left.as_string(),
-                self.reverse_op[operator],
-                right.as_string(),
+            suggestion = (
+                f"{left.as_string()} {self.reverse_op[operator]} {right.as_string()}"
             )
             self.add_message(
                 "unneeded-not", node=node, args=(node.as_string(), suggestion)
