@@ -1,36 +1,11 @@
-# Copyright (c) 2006, 2008-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2012 Ry4an Brase <ry4an-hg@ry4an.org>
-# Copyright (c) 2012 Google, Inc.
-# Copyright (c) 2012 Anthony VEREZ <anthony.verez.external@cassidian.com>
-# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2014 Brett Cannon <brett@python.org>
-# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
-# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
-# Copyright (c) 2017, 2020 Anthony Sottile <asottile@umich.edu>
-# Copyright (c) 2017 Mikhail Fesenko <proggga@gmail.com>
-# Copyright (c) 2018 Scott Worley <scottworley@scottworley.com>
-# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
-# Copyright (c) 2019, 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
-# Copyright (c) 2019 Taewon D. Kim <kimt33@mcmaster.ca>
-# Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2020 Frank Harrison <frank@doublethefish.com>
-# Copyright (c) 2020 Eli Fine <ejfine@gmail.com>
-# Copyright (c) 2020 Shiv Venkatasubrahmanyam <shvenkat@users.noreply.github.com>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-# Copyright (c) 2021 Maksym Humetskyi <Humetsky@gmail.com>
-# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
-# Copyright (c) 2021 bot <bot@noreply.github.com>
-# Copyright (c) 2021 Aditya Gupta <adityagupta1089@users.noreply.github.com>
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
-# pylint: disable=redefined-builtin
-"""a similarities / code duplication command line tool and pylint checker
+"""A similarities / code duplication command line tool and pylint checker.
 
 The algorithm is based on comparing the hash value of n successive lines of a file.
-First the files are read and any line that doesn't fullfill requirement are removed (comments, docstrings...)
+First the files are read and any line that doesn't fulfill requirement are removed (comments, docstrings...)
 Those stripped lines are stored in the LineSet class which gives access to them.
 Then each index of the stripped lines collection is associated with the hash of n successive entries of the stripped lines starting at the current index
 (n is the minimum common lines option).
@@ -38,7 +13,7 @@ The common hashes between both linesets are then looked for. If there are matche
 with the corresponding couples (start line number/end line number) in both files.
 This association is then postprocessed to handle the case of successive matches. For example if the minimum common lines setting is set to four, then
 the hashes are computed with four lines. If one of match indices couple (12, 34) is the successor of another one (11, 33) then it means that there are
-in fact five lines wich are common.
+in fact five lines which are common.
 Once postprocessed the values of association table are the result looked for, i.e start and end lines numbers of common lines in both files.
 """
 import copy
@@ -47,12 +22,15 @@ import itertools
 import operator
 import re
 import sys
+import warnings
 from collections import defaultdict
 from getopt import getopt
-from io import TextIOWrapper
+from io import BufferedIOBase, BufferedReader, BytesIO
 from itertools import chain, groupby
 from typing import (
+    TYPE_CHECKING,
     Any,
+    Callable,
     Dict,
     FrozenSet,
     Generator,
@@ -60,8 +38,11 @@ from typing import (
     List,
     NamedTuple,
     NewType,
+    Optional,
     Set,
+    TextIO,
     Tuple,
+    Union,
 )
 
 import astroid
@@ -70,7 +51,10 @@ from astroid import nodes
 from pylint.checkers import BaseChecker, MapReduceMixin, table_lines_from_stats
 from pylint.interfaces import IRawChecker
 from pylint.reporters.ureports.nodes import Table
-from pylint.utils import decoding_stream
+from pylint.utils import LinterStats, decoding_stream
+
+if TYPE_CHECKING:
+    from pylint.lint import PyLinter
 
 DEFAULT_MIN_SIMILARITY_LINE = 4
 
@@ -90,18 +74,19 @@ class LineSpecifs(NamedTuple):
 
 
 # Links LinesChunk object to the starting indices (in lineset's stripped lines)
-# of the different chunk of lines that are used to compute the hash
+# of the different chunk of lines that are used to compute the hash
 HashToIndex_T = Dict["LinesChunk", List[Index]]
 
 # Links index in the lineset's stripped lines to the real lines in the file
 IndexToLines_T = Dict[Index, "SuccessiveLinesLimits"]
 
+# The types the streams read by pylint can take. Originating from astroid.nodes.Module.stream() and open()
+STREAM_TYPES = Union[TextIO, BufferedReader, BytesIO]
+
 
 class CplSuccessiveLinesLimits:
-    """
-    This class holds a couple of SuccessiveLinesLimits objects, one for each file compared,
-    and a counter on the number of common lines between both stripped lines collections extracted
-    from both files
+    """Holds a SuccessiveLinesLimits object for each file compared and a
+    counter on the number of common lines between both stripped lines collections extracted from both files
     """
 
     __slots__ = ("first_file", "second_file", "effective_cmn_lines_nb")
@@ -117,27 +102,25 @@ class CplSuccessiveLinesLimits:
         self.effective_cmn_lines_nb = effective_cmn_lines_nb
 
 
-# Links the indices ot the starting line in both lineset's stripped lines to
+# Links the indices to the starting line in both lineset's stripped lines to
 # the start and end lines in both files
 CplIndexToCplLines_T = Dict["LineSetStartCouple", CplSuccessiveLinesLimits]
 
 
 class LinesChunk:
-    """
-    The LinesChunk object computes and stores the hash of some consecutive stripped lines of a lineset.
-    """
+    """The LinesChunk object computes and stores the hash of some consecutive stripped lines of a lineset."""
 
     __slots__ = ("_fileid", "_index", "_hash")
 
     def __init__(self, fileid: str, num_line: int, *lines: Iterable[str]) -> None:
         self._fileid: str = fileid
-        """The name of the file from which the LinesChunk object is generated """
+        """The name of the file from which the LinesChunk object is generated."""
 
         self._index: Index = Index(num_line)
-        """The index in the stripped lines that is the starting of consecutive lines"""
+        """The index in the stripped lines that is the starting of consecutive lines."""
 
         self._hash: int = sum(hash(lin) for lin in lines)
-        """The hash of some consecutive lines"""
+        """The hash of some consecutive lines."""
 
     def __eq__(self, o: Any) -> bool:
         if not isinstance(o, LinesChunk):
@@ -160,8 +143,7 @@ class LinesChunk:
 
 
 class SuccessiveLinesLimits:
-    """
-    A class to handle the numbering of begin and end of successive lines.
+    """A class to handle the numbering of begin and end of successive lines.
 
     :note: Only the end line number can be updated.
     """
@@ -189,9 +171,7 @@ class SuccessiveLinesLimits:
 
 
 class LineSetStartCouple(NamedTuple):
-    """
-    Indices in both linesets that mark the beginning of successive lines
-    """
+    """Indices in both linesets that mark the beginning of successive lines."""
 
     fst_lineset_index: Index
     snd_lineset_index: Index
@@ -225,8 +205,9 @@ LinesChunkLimits_T = Tuple["LineSet", LineNumber, LineNumber]
 def hash_lineset(
     lineset: "LineSet", min_common_lines: int = DEFAULT_MIN_SIMILARITY_LINE
 ) -> Tuple[HashToIndex_T, IndexToLines_T]:
-    """
-    Return two dicts. The first associates the hash of successive stripped lines of a lineset
+    """Return two dicts.
+
+    The first associates the hash of successive stripped lines of a lineset
     to the indices of the starting lines.
     The second dict, associates the index of the starting line in the lineset's stripped lines to the
     couple [start, end] lines number in the corresponding file.
@@ -265,8 +246,7 @@ def hash_lineset(
 
 
 def remove_successives(all_couples: CplIndexToCplLines_T) -> None:
-    """
-    Removes all successive entries in the dictionary in argument
+    """Removes all successive entries in the dictionary in argument.
 
     :param all_couples: collection that has to be cleaned up from successives entries.
                         The keys are couples of indices that mark the beginning of common entries
@@ -315,10 +295,12 @@ def filter_noncode_lines(
     stindex_2: Index,
     common_lines_nb: int,
 ) -> int:
-    """
-    Return the effective number of common lines between lineset1 and lineset2 filtered from non code lines, that is to say the number of
-    common successive stripped lines except those that do not contain code (for example a ligne with only an
-    ending parathensis)
+    """Return the effective number of common lines between lineset1
+    and lineset2 filtered from non code lines.
+
+    That is to say the number of common successive stripped
+    lines except those that do not contain code (for example
+    a line with only an ending parathensis)
 
     :param ls_1: first lineset
     :param stindex_1: first lineset starting index
@@ -351,7 +333,7 @@ class Commonality(NamedTuple):
 
 
 class Similar:
-    """finds copy-pasted lines of code in a project"""
+    """Finds copy-pasted lines of code in a project."""
 
     def __init__(
         self,
@@ -369,33 +351,43 @@ class Similar:
         self.linesets: List["LineSet"] = []
 
     def append_stream(
-        self, streamid: str, stream: TextIOWrapper, encoding=None
+        self, streamid: str, stream: STREAM_TYPES, encoding: Optional[str] = None
     ) -> None:
-        """append a file to search for similarities"""
-        if encoding is None:
-            readlines = stream.readlines
-        else:
+        """Append a file to search for similarities."""
+        if isinstance(stream, BufferedIOBase):
+            if encoding is None:
+                raise ValueError
             readlines = decoding_stream(stream, encoding).readlines
+        else:
+            readlines = stream.readlines  # type: ignore[assignment] # hint parameter is incorrectly typed as non-optional
+
         try:
-            self.linesets.append(
-                LineSet(
-                    streamid,
-                    readlines(),
-                    self.ignore_comments,
-                    self.ignore_docstrings,
-                    self.ignore_imports,
-                    self.ignore_signatures,
-                )
-            )
+            lines = readlines()
         except UnicodeDecodeError:
-            pass
+            lines = []
+
+        self.linesets.append(
+            LineSet(
+                streamid,
+                lines,
+                self.ignore_comments,
+                self.ignore_docstrings,
+                self.ignore_imports,
+                self.ignore_signatures,
+                line_enabled_callback=self.linter._is_one_message_enabled  # type: ignore[attr-defined]
+                if hasattr(self, "linter")
+                else None,
+            )
+        )
 
     def run(self) -> None:
-        """start looking for similarities and display results on stdout"""
+        """Start looking for similarities and display results on stdout."""
+        if self.min_lines == 0:
+            return
         self._display_sims(self._compute_sims())
 
     def _compute_sims(self) -> List[Tuple[int, Set[LinesChunkLimits_T]]]:
-        """compute similarities in appended files"""
+        """Compute similarities in appended files."""
         no_duplicates: Dict[int, List[Set[LinesChunkLimits_T]]] = defaultdict(list)
 
         for commonality in self._iter_sims():
@@ -436,14 +428,14 @@ class Similar:
     def _display_sims(
         self, similarities: List[Tuple[int, Set[LinesChunkLimits_T]]]
     ) -> None:
-        """Display computed similarities on stdout"""
+        """Display computed similarities on stdout."""
         report = self._get_similarity_report(similarities)
         print(report)
 
     def _get_similarity_report(
         self, similarities: List[Tuple[int, Set[LinesChunkLimits_T]]]
     ) -> str:
-        """Create a report from similarities"""
+        """Create a report from similarities."""
         report: str = ""
         duplicated_line_number: int = 0
         for number, couples in similarities:
@@ -457,18 +449,13 @@ class Similar:
                     report += f"   {line.rstrip()}\n" if line.rstrip() else "\n"
             duplicated_line_number += number * (len(couples_l) - 1)
         total_line_number: int = sum(len(lineset) for lineset in self.linesets)
-        report += "TOTAL lines={} duplicates={} percent={:.2f}\n".format(
-            total_line_number,
-            duplicated_line_number,
-            duplicated_line_number * 100.0 / total_line_number,
-        )
+        report += f"TOTAL lines={total_line_number} duplicates={duplicated_line_number} percent={duplicated_line_number * 100.0 / total_line_number:.2f}\n"
         return report
 
     def _find_common(
         self, lineset1: "LineSet", lineset2: "LineSet"
     ) -> Generator[Commonality, None, None]:
-        """
-        Find similarities in the two given linesets.
+        """Find similarities in the two given linesets.
 
         This the core of the algorithm.
         The idea is to compute the hashes of a minimal number of successive lines of each lineset and then compare the hashes.
@@ -534,7 +521,7 @@ class Similar:
                 yield com
 
     def _iter_sims(self) -> Generator[Commonality, None, None]:
-        """iterate on similarities among all files, by making a cartesian
+        """Iterate on similarities among all files, by making a cartesian
         product
         """
         for idx, lineset in enumerate(self.linesets[:-1]):
@@ -542,7 +529,7 @@ class Similar:
                 yield from self._find_common(lineset, lineset2)
 
     def get_map_data(self):
-        """Returns the data we can use for a map/reduce process
+        """Returns the data we can use for a map/reduce process.
 
         In this case we are returning this instance's Linesets, that is all file
         information that will later be used for vectorisation.
@@ -550,9 +537,10 @@ class Similar:
         return self.linesets
 
     def combine_mapreduce_data(self, linesets_collection):
-        """Reduces and recombines data into a format that we can report on
+        """Reduces and recombines data into a format that we can report on.
 
-        The partner function of get_map_data()"""
+        The partner function of get_map_data()
+        """
         self.linesets = [line for lineset in linesets_collection for line in lineset]
 
 
@@ -562,15 +550,16 @@ def stripped_lines(
     ignore_docstrings: bool,
     ignore_imports: bool,
     ignore_signatures: bool,
+    line_enabled_callback: Union[Callable[[str, int], bool], None] = None,
 ) -> List[LineSpecifs]:
-    """
-    Return tuples of line/line number/line type with leading/trailing whitespace and any ignored code features removed
+    """Return tuples of line/line number/line type with leading/trailing whitespace and any ignored code features removed.
 
     :param lines: a collection of lines
     :param ignore_comments: if true, any comment in the lines collection is removed from the result
     :param ignore_docstrings: if true, any line that is a docstring is removed from the result
     :param ignore_imports: if true, any line that is an import is removed from the result
     :param ignore_signatures: if true, any line that is part of a function signature is removed from the result
+    :param line_enabled_callback: If called with "R0801" and a line number, a return value of False will disregard the line
     :return: the collection of line/line number/line type tuples
     """
     if ignore_imports or ignore_signatures:
@@ -622,6 +611,10 @@ def stripped_lines(
     strippedlines = []
     docstring = None
     for lineno, line in enumerate(lines, start=1):
+        if line_enabled_callback is not None and not line_enabled_callback(
+            "R0801", lineno
+        ):
+            continue
         line = line.strip()
         if ignore_docstrings:
             if not docstring:
@@ -654,29 +647,35 @@ def stripped_lines(
 
 @functools.total_ordering
 class LineSet:
-    """
-    Holds and indexes all the lines of a single source file.
-    Allows for correspondance between real lines of the source file and stripped ones, which
+    """Holds and indexes all the lines of a single source file.
+
+    Allows for correspondence between real lines of the source file and stripped ones, which
     are the real ones from which undesired patterns have been removed.
     """
 
     def __init__(
         self,
-        name,
-        lines,
-        ignore_comments=False,
-        ignore_docstrings=False,
-        ignore_imports=False,
-        ignore_signatures=False,
+        name: str,
+        lines: List[str],
+        ignore_comments: bool = False,
+        ignore_docstrings: bool = False,
+        ignore_imports: bool = False,
+        ignore_signatures: bool = False,
+        line_enabled_callback: Union[Callable[[str, int], bool], None] = None,
     ) -> None:
         self.name = name
         self._real_lines = lines
         self._stripped_lines = stripped_lines(
-            lines, ignore_comments, ignore_docstrings, ignore_imports, ignore_signatures
+            lines,
+            ignore_comments,
+            ignore_docstrings,
+            ignore_imports,
+            ignore_signatures,
+            line_enabled_callback=line_enabled_callback,
         )
 
     def __str__(self):
-        return "<Lineset for %s>" % self.name
+        return f"<Lineset for {self.name}>"
 
     def __len__(self):
         return len(self._real_lines)
@@ -715,20 +714,23 @@ MSGS = {
 }
 
 
-def report_similarities(sect, stats, old_stats):
-    """make a layout with some stats about duplication"""
+def report_similarities(
+    sect,
+    stats: LinterStats,
+    old_stats: Optional[LinterStats],
+) -> None:
+    """Make a layout with some stats about duplication."""
     lines = ["", "now", "previous", "difference"]
-    lines += table_lines_from_stats(
-        stats, old_stats, ("nb_duplicated_lines", "percent_duplicated_lines")
-    )
+    lines += table_lines_from_stats(stats, old_stats, "duplicated_lines")
     sect.append(Table(children=lines, cols=4, rheaders=1, cheaders=1))
 
 
 # wrapper to get a pylint checker from the similar class
 class SimilarChecker(BaseChecker, Similar, MapReduceMixin):
-    """checks for similarities and duplicated code. This computation may be
-    memory / CPU intensive, so you should disable it if you experiment some
-    problems.
+    """Checks for similarities and duplicated code.
+
+    This computation may be memory / CPU intensive, so you
+    should disable it if you experiment some problems.
     """
 
     __implements__ = (IRawChecker,)
@@ -740,7 +742,7 @@ class SimilarChecker(BaseChecker, Similar, MapReduceMixin):
     # for available dict keys/values see the optik parser 'add_option' method
     options = (
         (
-            "min-similarity-lines",  # type: ignore
+            "min-similarity-lines",
             {
                 "default": DEFAULT_MIN_SIMILARITY_LINE,
                 "type": "int",
@@ -786,7 +788,7 @@ class SimilarChecker(BaseChecker, Similar, MapReduceMixin):
         ),
     )
     # reports
-    reports = (("RP0801", "Duplication", report_similarities),)  # type: ignore
+    reports = (("RP0801", "Duplication", report_similarities),)
 
     def __init__(self, linter=None) -> None:
         BaseChecker.__init__(self, linter)
@@ -798,10 +800,9 @@ class SimilarChecker(BaseChecker, Similar, MapReduceMixin):
             ignore_imports=self.config.ignore_imports,
             ignore_signatures=self.config.ignore_signatures,
         )
-        self.stats = None
 
     def set_option(self, optname, value, action=None, optdict=None):
-        """method called to set an option (registered in the options list)
+        """Method called to set an option (registered in the options list).
 
         Overridden to report options setting to Similar
         """
@@ -818,27 +819,33 @@ class SimilarChecker(BaseChecker, Similar, MapReduceMixin):
             self.ignore_signatures = self.config.ignore_signatures
 
     def open(self):
-        """init the checkers: reset linesets and statistics information"""
+        """Init the checkers: reset linesets and statistics information."""
         self.linesets = []
-        self.stats = self.linter.add_stats(
-            nb_duplicated_lines=0, percent_duplicated_lines=0
-        )
+        self.linter.stats.reset_duplicated_lines()
 
-    def process_module(self, node):
-        """process a module
+    def process_module(self, node: nodes.Module) -> None:
+        """Process a module.
 
         the module's content is accessible via the stream object
 
         stream must implement the readlines method
         """
+        if self.linter.current_name is None:
+            warnings.warn(
+                (
+                    "In pylint 3.0 the current_name attribute of the linter object should be a string. "
+                    "If unknown it should be initialized as an empty string."
+                ),
+                DeprecationWarning,
+            )
         with node.stream() as stream:
-            self.append_stream(self.linter.current_name, stream, node.file_encoding)
+            self.append_stream(self.linter.current_name, stream, node.file_encoding)  # type: ignore[arg-type]
 
     def close(self):
-        """compute and display similarities on closing (i.e. end of parsing)"""
+        """Compute and display similarities on closing (i.e. end of parsing)."""
         total = sum(len(lineset) for lineset in self.linesets)
         duplicated = 0
-        stats = self.stats
+        stats = self.linter.stats
         for num, couples in self._compute_sims():
             msg = []
             lineset = start_line = end_line = None
@@ -852,17 +859,18 @@ class SimilarChecker(BaseChecker, Similar, MapReduceMixin):
 
             self.add_message("R0801", args=(len(couples), "\n".join(msg)))
             duplicated += num * (len(couples) - 1)
-        stats["nb_duplicated_lines"] = duplicated
-        stats["percent_duplicated_lines"] = total and duplicated * 100.0 / total
+        stats.nb_duplicated_lines += int(duplicated)
+        stats.percent_duplicated_lines += float(total and duplicated * 100.0 / total)
 
     def get_map_data(self):
-        """Passthru override"""
+        """Passthru override."""
         return Similar.get_map_data(self)
 
     def reduce_map_data(self, linter, data):
-        """Reduces and recombines data into a format that we can report on
+        """Reduces and recombines data into a format that we can report on.
 
-        The partner function of get_map_data()"""
+        The partner function of get_map_data()
+        """
         recombined = SimilarChecker(linter)
         recombined.min_lines = self.min_lines
         recombined.ignore_comments = self.ignore_comments
@@ -874,13 +882,12 @@ class SimilarChecker(BaseChecker, Similar, MapReduceMixin):
         recombined.close()
 
 
-def register(linter):
-    """required method to auto register this checker"""
+def register(linter: "PyLinter") -> None:
     linter.register_checker(SimilarChecker(linter))
 
 
 def usage(status=0):
-    """display command line usage information"""
+    """Display command line usage information."""
     print("finds copy pasted blocks in a set of files")
     print()
     print(
@@ -891,7 +898,7 @@ def usage(status=0):
 
 
 def Run(argv=None):
-    """standalone command line access point"""
+    """Standalone command line access point."""
     if argv is None:
         argv = sys.argv[1:]
 
@@ -911,17 +918,17 @@ def Run(argv=None):
     ignore_signatures = False
     opts, args = getopt(argv, s_opts, l_opts)
     for opt, val in opts:
-        if opt in ("-d", "--duplicates"):
+        if opt in {"-d", "--duplicates"}:
             min_lines = int(val)
-        elif opt in ("-h", "--help"):
+        elif opt in {"-h", "--help"}:
             usage()
-        elif opt in ("-i", "--ignore-comments"):
+        elif opt in {"-i", "--ignore-comments"}:
             ignore_comments = True
-        elif opt in ("--ignore-docstrings",):
+        elif opt in {"--ignore-docstrings"}:
             ignore_docstrings = True
-        elif opt in ("--ignore-imports",):
+        elif opt in {"--ignore-imports"}:
             ignore_imports = True
-        elif opt in ("--ignore-signatures",):
+        elif opt in {"--ignore-signatures"}:
             ignore_signatures = True
     if not args:
         usage(1)
