@@ -1,67 +1,16 @@
-# Copyright (c) 2006-2007, 2009-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2009 Mads Kiilerich <mads@kiilerich.com>
-# Copyright (c) 2010 Daniel Harding <dharding@gmail.com>
-# Copyright (c) 2012-2014 Google, Inc.
-# Copyright (c) 2012 FELD Boris <lothiraldan@gmail.com>
-# Copyright (c) 2013-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2014 Brett Cannon <brett@python.org>
-# Copyright (c) 2014 Ricardo Gemignani <ricardo.gemignani@gmail.com>
-# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
-# Copyright (c) 2015 Dmitry Pribysh <dmand@yandex.ru>
-# Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
-# Copyright (c) 2015 Radu Ciorba <radu@devrandom.ro>
-# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
-# Copyright (c) 2016, 2018-2019 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2016-2017 Łukasz Rogalski <rogalski.91@gmail.com>
-# Copyright (c) 2016-2017 Moises Lopez <moylop260@vauxoo.com>
-# Copyright (c) 2016 Brian C. Lane <bcl@redhat.com>
-# Copyright (c) 2017-2018, 2020 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2017 ttenhoeve-aa <ttenhoeve@appannie.com>
-# Copyright (c) 2018 Alan Chan <achan961117@gmail.com>
-# Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
-# Copyright (c) 2018 Yury Gribov <tetra2005@gmail.com>
-# Copyright (c) 2018 Caio Carrara <ccarrara@redhat.com>
-# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
-# Copyright (c) 2018 Bryce Guinta <bryce.guinta@protonmail.com>
-# Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
-# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
-# Copyright (c) 2018 Brian Shaginaw <brian.shaginaw@warbyparker.com>
-# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2019 Matthijs Blom <19817960+MatthijsBlom@users.noreply.github.com>
-# Copyright (c) 2019 Djailla <bastien.vallet@gmail.com>
-# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
-# Copyright (c) 2019 Nathan Marrow <nmarrow@google.com>
-# Copyright (c) 2019 Svet <svet@hyperscience.com>
-# Copyright (c) 2019 Pascal Corpet <pcorpet@users.noreply.github.com>
-# Copyright (c) 2020 Batuhan Taskaya <batuhanosmantaskaya@gmail.com>
-# Copyright (c) 2020 Luigi <luigi.cristofolini@q-ctrl.com>
-# Copyright (c) 2020 ethan-leba <ethanleba5@gmail.com>
-# Copyright (c) 2020 Damien Baty <damien.baty@polyconseil.fr>
-# Copyright (c) 2020 Andrew Simmons <anjsimmo@gmail.com>
-# Copyright (c) 2020 Ram Rachum <ram@rachum.com>
-# Copyright (c) 2020 Slavfox <slavfoxman@gmail.com>
-# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-# Copyright (c) 2021 Nick Drozd <nicholasdrozd@gmail.com>
-# Copyright (c) 2021 David Liu <david@cs.toronto.edu>
-# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
-# Copyright (c) 2021 Matus Valo <matusvalo@users.noreply.github.com>
-# Copyright (c) 2021 Lorena B <46202743+lorena-b@users.noreply.github.com>
-# Copyright (c) 2021 yushao2 <36848472+yushao2@users.noreply.github.com>
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
-"""some functions that may be useful for various checkers
-"""
+"""Some functions that may be useful for various checkers."""
 import builtins
 import itertools
 import numbers
 import re
 import string
+import warnings
 from functools import lru_cache, partial
 from typing import (
-    Any,
     Callable,
     Dict,
     Iterable,
@@ -70,6 +19,8 @@ from typing import (
     Optional,
     Set,
     Tuple,
+    Type,
+    TypeVar,
     Union,
 )
 
@@ -79,6 +30,8 @@ import astroid.objects
 from astroid import TooManyLevelsError, nodes
 from astroid.context import InferenceContext
 
+from pylint.constants import TYPING_TYPE_CHECKS_GUARDS
+
 COMP_NODE_TYPES = (
     nodes.ListComp,
     nodes.SetComp,
@@ -112,7 +65,7 @@ KEYS_METHOD = "keys"
 #
 # * None: variable number of parameters
 # * number: exactly that number of parameters
-# * tuple: this are the odd ones. Basically it means that the function
+# * tuple: these are the odd ones. Basically it means that the function
 #          can work with any number of arguments from that tuple,
 #          although it's best to implement it in order to accept
 #          all of them.
@@ -154,6 +107,8 @@ _SPECIAL_METHODS_PARAMS = {
         "__aiter__",
         "__anext__",
         "__fspath__",
+        "__subclasses__",
+        "__init_subclass__",
     ),
     1: (
         "__format__",
@@ -228,7 +183,7 @@ _SPECIAL_METHODS_PARAMS = {
 SPECIAL_METHODS_PARAMS = {
     name: params
     for params, methods in _SPECIAL_METHODS_PARAMS.items()
-    for name in methods  # type: ignore
+    for name in methods
 }
 PYMETHODS = set(SPECIAL_METHODS_PARAMS)
 
@@ -275,6 +230,8 @@ SUBSCRIPTABLE_CLASSES_PEP585 = frozenset(
     )
 )
 
+T_Node = TypeVar("T_Node", bound=nodes.NodeNG)
+
 
 class NoSuchArgumentError(Exception):
     pass
@@ -285,13 +242,13 @@ class InferredTypeError(Exception):
 
 
 def is_inside_lambda(node: nodes.NodeNG) -> bool:
-    """Return true if given node is inside lambda"""
-    parent = node.parent
-    while parent is not None:
-        if isinstance(parent, nodes.Lambda):
-            return True
-        parent = parent.parent
-    return False
+    """Return whether the given node is inside a lambda."""
+    warnings.warn(
+        "utils.is_inside_lambda will be removed in favour of calling "
+        "utils.get_node_first_ancestor_of_type(x, nodes.Lambda) in pylint 3.0",
+        DeprecationWarning,
+    )
+    return any(isinstance(parent, nodes.Lambda) for parent in node.node_ancestors())
 
 
 def get_all_elements(
@@ -306,18 +263,18 @@ def get_all_elements(
 
 
 def is_super(node: nodes.NodeNG) -> bool:
-    """return True if the node is referencing the "super" builtin function"""
+    """Return True if the node is referencing the "super" builtin function."""
     if getattr(node, "name", None) == "super" and node.root().name == "builtins":
         return True
     return False
 
 
 def is_error(node: nodes.FunctionDef) -> bool:
-    """Return true if the given function node only raises an exception"""
+    """Return true if the given function node only raises an exception."""
     return len(node.body) == 1 and isinstance(node.body[0], nodes.Raise)
 
 
-builtins = builtins.__dict__.copy()  # type: ignore
+builtins = builtins.__dict__.copy()  # type: ignore[assignment]
 SPECIAL_BUILTINS = ("__builtins__",)  # '__path__', '__file__')
 
 
@@ -327,8 +284,8 @@ def is_builtin_object(node: nodes.NodeNG) -> bool:
 
 
 def is_builtin(name: str) -> bool:
-    """return true if <name> could be considered as a builtin defined by python"""
-    return name in builtins or name in SPECIAL_BUILTINS  # type: ignore
+    """Return true if <name> could be considered as a builtin defined by python."""
+    return name in builtins or name in SPECIAL_BUILTINS  # type: ignore[attr-defined]
 
 
 def is_defined_in_scope(
@@ -380,7 +337,7 @@ def is_defined_in_scope(
 
 
 def is_defined_before(var_node: nodes.Name) -> bool:
-    """Check if the given variable node is defined before
+    """Check if the given variable node is defined before.
 
     Verify that the variable node is defined by a parent node
     (list, set, dict, or generator comprehension, lambda)
@@ -388,13 +345,11 @@ def is_defined_before(var_node: nodes.Name) -> bool:
     (statement_defining ; statement_using).
     """
     varname = var_node.name
-    _node = var_node.parent
-    while _node:
-        if is_defined_in_scope(var_node, varname, _node):
+    for parent in var_node.node_ancestors():
+        if is_defined_in_scope(var_node, varname, parent):
             return True
-        _node = _node.parent
-    # possibly multiple statements on the same line using semi colon separator
-    stmt = var_node.statement()
+    # possibly multiple statements on the same line using semicolon separator
+    stmt = var_node.statement(future=True)
     _node = stmt.previous_sibling()
     lineno = stmt.fromlineno
     while _node and _node.fromlineno == lineno:
@@ -411,23 +366,27 @@ def is_defined_before(var_node: nodes.Name) -> bool:
 def is_default_argument(
     node: nodes.NodeNG, scope: Optional[nodes.NodeNG] = None
 ) -> bool:
-    """return true if the given Name node is used in function or lambda
+    """Return true if the given Name node is used in function or lambda
     default argument's value
     """
     if not scope:
         scope = node.scope()
     if isinstance(scope, (nodes.FunctionDef, nodes.Lambda)):
-        for default_node in scope.args.defaults:
-            for default_name_node in default_node.nodes_of_class(nodes.Name):
-                if default_name_node is node:
-                    return True
+        all_defaults = itertools.chain(
+            scope.args.defaults, (d for d in scope.args.kw_defaults if d is not None)
+        )
+        return any(
+            default_name_node is node
+            for default_node in all_defaults
+            for default_name_node in default_node.nodes_of_class(nodes.Name)
+        )
+
     return False
 
 
 def is_func_decorator(node: nodes.NodeNG) -> bool:
-    """return true if the name is used in function decorator"""
-    parent = node.parent
-    while parent is not None:
+    """Return true if the name is used in function decorator."""
+    for parent in node.node_ancestors():
         if isinstance(parent, nodes.Decorators):
             return True
         if parent.is_statement or isinstance(
@@ -439,44 +398,44 @@ def is_func_decorator(node: nodes.NodeNG) -> bool:
             ),
         ):
             break
-        parent = parent.parent
     return False
 
 
 def is_ancestor_name(frame: nodes.ClassDef, node: nodes.NodeNG) -> bool:
-    """return True if `frame` is an astroid.Class node with `node` in the
+    """Return whether `frame` is an astroid.Class node with `node` in the
     subtree of its bases attribute
     """
     if not isinstance(frame, nodes.ClassDef):
         return False
-    for base in frame.bases:
-        if node in base.nodes_of_class(nodes.Name):
-            return True
-    return False
+    return any(node in base.nodes_of_class(nodes.Name) for base in frame.bases)
 
 
 def is_being_called(node: nodes.NodeNG) -> bool:
-    """return True if node is the function being called in a Call node"""
+    """Return True if node is the function being called in a Call node."""
     return isinstance(node.parent, nodes.Call) and node.parent.func is node
 
 
 def assign_parent(node: nodes.NodeNG) -> nodes.NodeNG:
-    """return the higher parent which is not an AssignName, Tuple or List node"""
+    """Return the higher parent which is not an AssignName, Tuple or List node."""
     while node and isinstance(node, (nodes.AssignName, nodes.Tuple, nodes.List)):
         node = node.parent
     return node
 
 
 def overrides_a_method(class_node: nodes.ClassDef, name: str) -> bool:
-    """return True if <name> is a method overridden from an ancestor"""
+    """Return True if <name> is a method overridden from an ancestor
+    which is not the base object class
+    """
     for ancestor in class_node.ancestors():
+        if ancestor.name == "object":
+            continue
         if name in ancestor and isinstance(ancestor[name], nodes.FunctionDef):
             return True
     return False
 
 
 def check_messages(*messages: str) -> Callable:
-    """decorator to store messages that are handled by a checker method"""
+    """Decorator to store messages that are handled by a checker method."""
 
     def store_messages(func):
         func.checks_msgs = messages
@@ -491,21 +450,23 @@ class IncompleteFormatString(Exception):
 
 class UnsupportedFormatCharacter(Exception):
     """A format character in a format string is not one of the supported
-    format characters."""
+    format characters.
+    """
 
     def __init__(self, index):
-        Exception.__init__(self, index)
+        super().__init__(index)
         self.index = index
 
 
 def parse_format_string(
     format_string: str,
 ) -> Tuple[Set[str], int, Dict[str, str], List[str]]:
-    """Parses a format string, returning a tuple of (keys, num_args), where keys
-    is the set of mapping keys in the format string, and num_args is the number
-    of arguments required by the format string.  Raises
-    IncompleteFormatString or UnsupportedFormatCharacter if a
-    parse error occurs."""
+    """Parses a format string, returning a tuple (keys, num_args).
+
+    Where 'keys' is the set of mapping keys in the format string, and 'num_args' is the number
+    of arguments required by the format string. Raises IncompleteFormatString or
+    UnsupportedFormatCharacter if a parse error occurs.
+    """
     keys = set()
     key_types = {}
     pos_types = []
@@ -582,8 +543,9 @@ def split_format_field_names(format_string) -> Tuple[str, Iterable[Tuple[bool, s
 
 def collect_string_fields(format_string) -> Iterable[Optional[str]]:
     """Given a format string, return an iterator
-    of all the valid format fields. It handles nested fields
-    as well.
+    of all the valid format fields.
+
+    It handles nested fields as well.
     """
     formatter = string.Formatter()
     try:
@@ -615,10 +577,10 @@ def collect_string_fields(format_string) -> Iterable[Optional[str]]:
 def parse_format_method_string(
     format_string: str,
 ) -> Tuple[List[Tuple[str, List[Tuple[bool, str]]]], int, int]:
-    """
-    Parses a PEP 3101 format string, returning a tuple of
-    (keyword_arguments, implicit_pos_args_cnt, explicit_pos_args),
-    where keyword_arguments is the set of mapping keys in the format string, implicit_pos_args_cnt
+    """Parses a PEP 3101 format string, returning a tuple of
+    (keyword_arguments, implicit_pos_args_cnt, explicit_pos_args).
+
+    keyword_arguments is the set of mapping keys in the format string, implicit_pos_args_cnt
     is the number of arguments required by the format string and
     explicit_pos_args is the number of arguments passed with the position.
     """
@@ -642,7 +604,7 @@ def parse_format_method_string(
 
 
 def is_attr_protected(attrname: str) -> bool:
-    """return True if attribute name is protected (start with _ and some other
+    """Return True if attribute name is protected (start with _ and some other
     details), False otherwise.
     """
     return (
@@ -653,12 +615,12 @@ def is_attr_protected(attrname: str) -> bool:
 
 
 def node_frame_class(node: nodes.NodeNG) -> Optional[nodes.ClassDef]:
-    """Return the class that is wrapping the given node
+    """Return the class that is wrapping the given node.
 
     The function returns a class for a method node (or a staticmethod or a
     classmethod), otherwise it returns `None`.
     """
-    klass = node.frame()
+    klass = node.frame(future=True)
     nodes_to_check = (
         nodes.NodeNG,
         astroid.UnboundMethod,
@@ -672,11 +634,18 @@ def node_frame_class(node: nodes.NodeNG) -> Optional[nodes.ClassDef]:
         if klass.parent is None:
             return None
 
-        klass = klass.parent.frame()
+        klass = klass.parent.frame(future=True)
 
     return klass
 
 
+def get_outer_class(class_node: astroid.ClassDef) -> Optional[astroid.ClassDef]:
+    """Return the class that is the outer class of given (nested) class_node."""
+    parent_klass = class_node.parent.frame(future=True)
+
+    return parent_klass if isinstance(parent_klass, astroid.ClassDef) else None
+
+
 def is_attr_private(attrname: str) -> Optional[Match[str]]:
     """Check that attribute name is private (at least two leading underscores,
     at most one trailing underscore)
@@ -686,7 +655,7 @@ def is_attr_private(attrname: str) -> Optional[Match[str]]:
 
 
 def get_argument_from_call(
-    call_node: nodes.Call, position: int = None, keyword: str = None
+    call_node: nodes.Call, position: Optional[int] = None, keyword: Optional[str] = None
 ) -> nodes.Name:
     """Returns the specified argument from a function call.
 
@@ -716,23 +685,19 @@ def get_argument_from_call(
 
 
 def inherit_from_std_ex(node: nodes.NodeNG) -> bool:
-    """
-    Return true if the given class node is subclass of
+    """Return whether the given class node is subclass of
     exceptions.Exception.
     """
     ancestors = node.ancestors() if hasattr(node, "ancestors") else []
-    for ancestor in itertools.chain([node], ancestors):
-        if (
-            ancestor.name in ("Exception", "BaseException")
-            and ancestor.root().name == EXCEPTIONS_MODULE
-        ):
-            return True
-    return False
+    return any(
+        ancestor.name in {"Exception", "BaseException"}
+        and ancestor.root().name == EXCEPTIONS_MODULE
+        for ancestor in itertools.chain([node], ancestors)
+    )
 
 
 def error_of_type(handler: nodes.ExceptHandler, error_type) -> bool:
-    """
-    Check if the given exception handler catches
+    """Check if the given exception handler catches
     the given error_type.
 
     The *handler* parameter is a node, representing an ExceptHandler node.
@@ -748,8 +713,8 @@ def error_of_type(handler: nodes.ExceptHandler, error_type) -> bool:
         return error
 
     if not isinstance(error_type, tuple):
-        error_type = (error_type,)  # type: ignore
-    expected_errors = {stringify_error(error) for error in error_type}  # type: ignore
+        error_type = (error_type,)
+    expected_errors = {stringify_error(error) for error in error_type}
     if not handler.type:
         return False
     return handler.catch(expected_errors)
@@ -779,24 +744,24 @@ def _is_property_kind(node, *kinds):
 
 
 def is_property_setter(node: nodes.FunctionDef) -> bool:
-    """Check if the given node is a property setter"""
+    """Check if the given node is a property setter."""
     return _is_property_kind(node, "setter")
 
 
 def is_property_deleter(node: nodes.FunctionDef) -> bool:
-    """Check if the given node is a property deleter"""
+    """Check if the given node is a property deleter."""
     return _is_property_kind(node, "deleter")
 
 
 def is_property_setter_or_deleter(node: nodes.FunctionDef) -> bool:
-    """Check if the given node is either a property setter or a deleter"""
+    """Check if the given node is either a property setter or a deleter."""
     return _is_property_kind(node, "setter", "deleter")
 
 
 def _is_property_decorator(decorator: nodes.Name) -> bool:
     for inferred in decorator.infer():
         if isinstance(inferred, nodes.ClassDef):
-            if inferred.qname() in ("builtins.property", "functools.cached_property"):
+            if inferred.qname() in {"builtins.property", "functools.cached_property"}:
                 return True
             for ancestor in inferred.ancestors():
                 if ancestor.name == "property" and ancestor.root().name == "builtins":
@@ -821,7 +786,9 @@ def _is_property_decorator(decorator: nodes.Name) -> bool:
 
 
 def decorated_with(
-    func: Union[nodes.FunctionDef, astroid.BoundMethod, astroid.UnboundMethod],
+    func: Union[
+        nodes.ClassDef, nodes.FunctionDef, astroid.BoundMethod, astroid.UnboundMethod
+    ],
     qnames: Iterable[str],
 ) -> bool:
     """Determine if the `func` node has a decorator with the qualified name `qname`."""
@@ -842,12 +809,56 @@ def decorated_with(
     return False
 
 
+def uninferable_final_decorators(
+    node: nodes.Decorators,
+) -> List[Optional[Union[nodes.Attribute, nodes.Name]]]:
+    """Return a list of uninferable `typing.final` decorators in `node`.
+
+    This function is used to determine if the `typing.final` decorator is used
+    with an unsupported Python version; the decorator cannot be inferred when
+    using a Python version lower than 3.8.
+    """
+    decorators = []
+    for decorator in getattr(node, "nodes", []):
+        if isinstance(decorator, nodes.Attribute):
+            try:
+                import_node = decorator.expr.lookup(decorator.expr.name)[1][0]
+            except AttributeError:
+                continue
+        elif isinstance(decorator, nodes.Name):
+            lookup_values = decorator.lookup(decorator.name)
+            if lookup_values[1]:
+                import_node = lookup_values[1][0]
+            else:
+                continue  # pragma: no cover # Covered on Python < 3.8
+        else:
+            continue
+
+        if not isinstance(import_node, (astroid.Import, astroid.ImportFrom)):
+            continue
+
+        import_names = dict(import_node.names)
+
+        # from typing import final
+        is_from_import = ("final" in import_names) and import_node.modname == "typing"
+        # import typing
+        is_import = ("typing" in import_names) and getattr(
+            decorator, "attrname", None
+        ) == "final"
+
+        if (is_from_import or is_import) and safe_infer(decorator) in [
+            astroid.Uninferable,
+            None,
+        ]:
+            decorators.append(decorator)
+    return decorators
+
+
 @lru_cache(maxsize=1024)
 def unimplemented_abstract_methods(
     node: nodes.ClassDef, is_abstract_cb: nodes.FunctionDef = None
 ) -> Dict[str, nodes.NodeNG]:
-    """
-    Get the unimplemented abstract methods for the given *node*.
+    """Get the unimplemented abstract methods for the given *node*.
 
     A method can be considered abstract if the callback *is_abstract_cb*
     returns a ``True`` value. The check defaults to verifying that
@@ -866,8 +877,7 @@ def unimplemented_abstract_methods(
         # Old style class, it will not have a mro.
         return {}
     except astroid.ResolveError:
-        # Probably inconsistent hierarchy, don'try
-        # to figure this out here.
+        # Probably inconsistent hierarchy, don't try to figure this out here.
         return {}
     for ancestor in mro:
         for obj in ancestor.values():
@@ -920,9 +930,7 @@ def find_except_wrapper_node_in_scope(
     node: nodes.NodeNG,
 ) -> Optional[Union[nodes.ExceptHandler, nodes.TryExcept]]:
     """Return the ExceptHandler in which the node is, without going out of scope."""
-    current = node
-    while current.parent is not None:
-        current = current.parent
+    for current in node.node_ancestors():
         if isinstance(current, astroid.scoped_nodes.LocalsDictNodeNG):
             # If we're inside a function/class definition, we don't want to keep checking
             # higher ancestors for `except` clauses, because if these exist, it means our
@@ -953,14 +961,17 @@ def is_from_fallback_block(node: nodes.NodeNG) -> bool:
         isinstance(import_node, (nodes.ImportFrom, nodes.Import))
         for import_node in other_body
     )
-    ignores_import_error = _except_handlers_ignores_exception(handlers, ImportError)
+    ignores_import_error = _except_handlers_ignores_exceptions(
+        handlers, (ImportError, ModuleNotFoundError)
+    )
     return ignores_import_error or has_fallback_imports
 
 
-def _except_handlers_ignores_exception(
-    handlers: nodes.ExceptHandler, exception
+def _except_handlers_ignores_exceptions(
+    handlers: nodes.ExceptHandler,
+    exceptions: Tuple[Type[ImportError], Type[ModuleNotFoundError]],
 ) -> bool:
-    func = partial(error_of_type, error_type=(exception,))
+    func = partial(error_of_type, error_type=exceptions)
     return any(func(handler) for handler in handlers)
 
 
@@ -986,8 +997,8 @@ def get_exception_handlers(
 
 
 def is_node_inside_try_except(node: nodes.Raise) -> bool:
-    """Check if the node is directly under a Try/Except statement.
-    (but not under an ExceptHandler!)
+    """Check if the node is directly under a Try/Except statement
+    (but not under an ExceptHandler!).
 
     Args:
         node (nodes.Raise): the node raising the exception.
@@ -1012,7 +1023,7 @@ def node_ignores_exception(node: nodes.NodeNG, exception=Exception) -> bool:
 
 
 def class_is_abstract(node: nodes.ClassDef) -> bool:
-    """return true if the given class node should be considered as an abstract
+    """Return true if the given class node should be considered as an abstract
     class
     """
     # Only check for explicit metaclass=ABCMeta on this specific class
@@ -1027,7 +1038,7 @@ def class_is_abstract(node: nodes.ClassDef) -> bool:
             return True
 
     for method in node.methods():
-        if method.parent.frame() is node:
+        if method.parent.frame(future=True) is node:
             if method.is_abstract(pass_is_abstract=False):
                 return True
     return False
@@ -1040,8 +1051,19 @@ def _supports_protocol_method(value: nodes.NodeNG, attr: str) -> bool:
         return False
 
     first = attributes[0]
+
+    # Return False if a constant is assigned
     if isinstance(first, nodes.AssignName):
-        if isinstance(first.parent.value, nodes.Const):
+        this_assign_parent = get_node_first_ancestor_of_type(
+            first, (nodes.Assign, nodes.NamedExpr)
+        )
+        if this_assign_parent is None:  # pragma: no cover
+            # Cannot imagine this being None, but return True to avoid false positives
+            return True
+        if isinstance(this_assign_parent.value, nodes.BaseContainer):
+            if all(isinstance(n, nodes.Const) for n in this_assign_parent.value.elts):
+                return False
+        if isinstance(this_assign_parent.value, nodes.Const):
             return False
     return True
 
@@ -1127,6 +1149,11 @@ def _supports_protocol(
         if protocol_callback(value):
             return True
 
+    # pylint: disable-next=fixme
+    # TODO: Should be covered by https://github.com/PyCQA/astroid/pull/1475
+    if isinstance(value, nodes.ComprehensionScope):
+        return True
+
     if (
         isinstance(value, astroid.bases.Proxy)
         and isinstance(value._proxied, astroid.BaseInstance)
@@ -1164,15 +1191,15 @@ def supports_getitem(value: nodes.NodeNG, node: nodes.NodeNG) -> bool:
     return _supports_protocol(value, _supports_getitem_protocol)
 
 
-def supports_setitem(value: nodes.NodeNG, *_: Any) -> bool:
+def supports_setitem(value: nodes.NodeNG, _: nodes.NodeNG) -> bool:
     return _supports_protocol(value, _supports_setitem_protocol)
 
 
-def supports_delitem(value: nodes.NodeNG, *_: Any) -> bool:
+def supports_delitem(value: nodes.NodeNG, _: nodes.NodeNG) -> bool:
     return _supports_protocol(value, _supports_delitem_protocol)
 
 
-def _get_python_type_of_node(node):
+def _get_python_type_of_node(node: nodes.NodeNG) -> Optional[str]:
     pytype = getattr(node, "pytype", None)
     if callable(pytype):
         return pytype()
@@ -1180,13 +1207,15 @@ def _get_python_type_of_node(node):
 
 
 @lru_cache(maxsize=1024)
-def safe_infer(node: nodes.NodeNG, context=None) -> Optional[nodes.NodeNG]:
+def safe_infer(
+    node: nodes.NodeNG, context: Optional[InferenceContext] = None
+) -> Union[nodes.NodeNG, Type[astroid.Uninferable], None]:
     """Return the inferred value for the given node.
 
     Return None if inference failed or if there is some ambiguity (more than
     one node has been inferred of different types).
     """
-    inferred_types = set()
+    inferred_types: Set[Optional[str]] = set()
     try:
         infer_gen = node.infer(context=context)
         value = next(infer_gen)
@@ -1201,6 +1230,14 @@ def safe_infer(node: nodes.NodeNG, context=None) -> Optional[nodes.NodeNG]:
             inferred_type = _get_python_type_of_node(inferred)
             if inferred_type not in inferred_types:
                 return None  # If there is ambiguity on the inferred node.
+            if (
+                isinstance(inferred, nodes.FunctionDef)
+                and inferred.args.args is not None
+                and isinstance(value, nodes.FunctionDef)
+                and value.args.args is not None
+                and len(inferred.args.args) != len(value.args.args)
+            ):
+                return None  # Different number of arguments indicates ambiguity
     except astroid.InferenceError:
         return None  # There is some kind of ambiguity
     except StopIteration:
@@ -1245,15 +1282,15 @@ def is_none(node: nodes.NodeNG) -> bool:
     )
 
 
-def node_type(node: nodes.NodeNG) -> Optional[type]:
-    """Return the inferred type for `node`
+def node_type(node: nodes.NodeNG) -> Optional[nodes.NodeNG]:
+    """Return the inferred type for `node`.
 
     If there is more than one possible type, or if inferred type is Uninferable or None,
     return None
     """
     # check there is only one possible type for the assign node. Else we
     # don't handle it for now
-    types = set()
+    types: Set[nodes.NodeNG] = set()
     try:
         for var_type in node.infer():
             if var_type == astroid.Uninferable or is_none(var_type):
@@ -1293,15 +1330,15 @@ def is_registered_in_singledispatch_function(node: nodes.FunctionDef) -> bool:
             continue
 
         if isinstance(func_def, nodes.FunctionDef):
-            # pylint: disable=redundant-keyword-arg; some flow inference goes wrong here
             return decorated_with(func_def, singledispatch_qnames)
 
     return False
 
 
 def get_node_last_lineno(node: nodes.NodeNG) -> int:
-    """
-    Get the last lineno of the given node. For a simple statement this will just be node.lineno,
+    """Get the last lineno of the given node.
+
+    For a simple statement this will just be node.lineno,
     but for a node that has child statements (e.g. a method) this will be the lineno of the last
     child statement recursively.
     """
@@ -1323,7 +1360,7 @@ def get_node_last_lineno(node: nodes.NodeNG) -> int:
 
 
 def is_postponed_evaluation_enabled(node: nodes.NodeNG) -> bool:
-    """Check if the postponed evaluation of annotations is enabled"""
+    """Check if the postponed evaluation of annotations is enabled."""
     module = node.root()
     return "annotations" in module.future_imports
 
@@ -1372,8 +1409,8 @@ def is_node_in_type_annotation_context(node: nodes.NodeNG) -> bool:
 
 
 def is_subclass_of(child: nodes.ClassDef, parent: nodes.ClassDef) -> bool:
-    """
-    Check if first node is a subclass of second node.
+    """Check if first node is a subclass of second node.
+
     :param child: Node to check for subclass.
     :param parent: Node to check for superclass.
     :returns: True if child is derived from parent. False otherwise.
@@ -1392,7 +1429,7 @@ def is_subclass_of(child: nodes.ClassDef, parent: nodes.ClassDef) -> bool:
 
 @lru_cache(maxsize=1024)
 def is_overload_stub(node: nodes.NodeNG) -> bool:
-    """Check if a node if is a function stub decorated with typing.overload.
+    """Check if a node is a function stub decorated with typing.overload.
 
     :param node: Node to check.
     :returns: True if node is an overload function stub. False otherwise.
@@ -1402,7 +1439,7 @@ def is_overload_stub(node: nodes.NodeNG) -> bool:
 
 
 def is_protocol_class(cls: nodes.NodeNG) -> bool:
-    """Check if the given node represents a protocol class
+    """Check if the given node represents a protocol class.
 
     :param cls: The node to check
     :returns: True if the node is a typing protocol class, false otherwise.
@@ -1416,7 +1453,7 @@ def is_protocol_class(cls: nodes.NodeNG) -> bool:
 
 
 def is_call_of_name(node: nodes.NodeNG, name: str) -> bool:
-    """Checks if node is a function call with the given name"""
+    """Checks if node is a function call with the given name."""
     return (
         isinstance(node, nodes.Call)
         and isinstance(node.func, nodes.Name)
@@ -1428,7 +1465,7 @@ def is_test_condition(
     node: nodes.NodeNG,
     parent: Optional[nodes.NodeNG] = None,
 ) -> bool:
-    """Returns true if the given node is being tested for truthiness"""
+    """Returns true if the given node is being tested for truthiness."""
     parent = parent or node.parent
     if isinstance(parent, (nodes.While, nodes.If, nodes.IfExp, nodes.Assert)):
         return node is parent.test or parent.test.parent_of(node)
@@ -1441,10 +1478,7 @@ def is_classdef_type(node: nodes.ClassDef) -> bool:
     """Test if ClassDef node is Type."""
     if node.name == "type":
         return True
-    for base in node.bases:
-        if isinstance(base, nodes.Name) and base.name == "type":
-            return True
-    return False
+    return any(isinstance(b, nodes.Name) and b.name == "type" for b in node.bases)
 
 
 def is_attribute_typed_annotation(
@@ -1523,8 +1557,7 @@ def get_iterating_dictionary_name(
 
 
 def get_subscript_const_value(node: nodes.Subscript) -> nodes.Const:
-    """
-    Returns the value 'subscript.slice' of a Subscript node.
+    """Returns the value 'subscript.slice' of a Subscript node.
 
     :param node: Subscript Node to extract value from
     :returns: Const Node containing subscript value
@@ -1540,7 +1573,7 @@ def get_subscript_const_value(node: nodes.Subscript) -> nodes.Const:
 def get_import_name(
     importnode: Union[nodes.Import, nodes.ImportFrom], modname: str
 ) -> str:
-    """Get a prepared module name from the given import node
+    """Get a prepared module name from the given import node.
 
     In the case of relative imports, this will return the
     absolute qualified module name, which might be useful
@@ -1562,10 +1595,138 @@ def get_import_name(
     return modname
 
 
+def is_sys_guard(node: nodes.If) -> bool:
+    """Return True if IF stmt is a sys.version_info guard.
+
+    >>> import sys
+    >>> if sys.version_info > (3, 8):
+    >>>     from typing import Literal
+    >>> else:
+    >>>     from typing_extensions import Literal
+    """
+    if isinstance(node.test, nodes.Compare):
+        value = node.test.left
+        if isinstance(value, nodes.Subscript):
+            value = value.value
+        if (
+            isinstance(value, nodes.Attribute)
+            and value.as_string() == "sys.version_info"
+        ):
+            return True
+
+    return False
+
+
+def is_typing_guard(node: nodes.If) -> bool:
+    """Return True if IF stmt is a typing guard.
+
+    >>> from typing import TYPE_CHECKING
+    >>> if TYPE_CHECKING:
+    >>>     from xyz import a
+    """
+    return isinstance(
+        node.test, (nodes.Name, nodes.Attribute)
+    ) and node.test.as_string().endswith("TYPE_CHECKING")
+
+
+def is_node_in_typing_guarded_import_block(node: nodes.NodeNG) -> bool:
+    """Return True if node is part for guarded `typing.TYPE_CHECKING` if block."""
+    return isinstance(node.parent, nodes.If) and is_typing_guard(node.parent)
+
+
 def is_node_in_guarded_import_block(node: nodes.NodeNG) -> bool:
     """Return True if node is part for guarded if block.
+
     I.e. `sys.version_info` or `typing.TYPE_CHECKING`
     """
     return isinstance(node.parent, nodes.If) and (
-        node.parent.is_sys_guard() or node.parent.is_typing_guard()
+        is_sys_guard(node.parent) or is_typing_guard(node.parent)
+    )
+
+
+def is_reassigned_after_current(node: nodes.NodeNG, varname: str) -> bool:
+    """Check if the given variable name is reassigned in the same scope after the current node."""
+    return any(
+        a.name == varname and a.lineno > node.lineno
+        for a in node.scope().nodes_of_class(
+            (nodes.AssignName, nodes.ClassDef, nodes.FunctionDef)
+        )
+    )
+
+
+def is_deleted_after_current(node: nodes.NodeNG, varname: str) -> bool:
+    """Check if the given variable name is deleted in the same scope after the current node."""
+    return any(
+        getattr(target, "name", None) == varname and target.lineno > node.lineno
+        for del_node in node.scope().nodes_of_class(nodes.Delete)
+        for target in del_node.targets
+    )
+
+
+def is_function_body_ellipsis(node: nodes.FunctionDef) -> bool:
+    """Checks whether a function body only consists of a single Ellipsis."""
+    return (
+        len(node.body) == 1
+        and isinstance(node.body[0], nodes.Expr)
+        and isinstance(node.body[0].value, nodes.Const)
+        and node.body[0].value.value == Ellipsis
+    )
+
+
+def is_base_container(node: Optional[nodes.NodeNG]) -> bool:
+    return isinstance(node, nodes.BaseContainer) and not node.elts
+
+
+def is_empty_dict_literal(node: Optional[nodes.NodeNG]) -> bool:
+    return isinstance(node, nodes.Dict) and not node.items
+
+
+def is_empty_str_literal(node: Optional[nodes.NodeNG]) -> bool:
+    return (
+        isinstance(node, nodes.Const) and isinstance(node.value, str) and not node.value
+    )
+
+
+def returns_bool(node: nodes.NodeNG) -> bool:
+    """Returns true if a node is a return that returns a constant boolean."""
+    return (
+        isinstance(node, nodes.Return)
+        and isinstance(node.value, nodes.Const)
+        and node.value.value in {True, False}
+    )
+
+
+def get_node_first_ancestor_of_type(
+    node: nodes.NodeNG, ancestor_type: Union[Type[T_Node], Tuple[Type[T_Node], ...]]
+) -> Optional[T_Node]:
+    """Return the first parent node that is any of the provided types (or None)."""
+    for ancestor in node.node_ancestors():
+        if isinstance(ancestor, ancestor_type):
+            return ancestor
+    return None
+
+
+def get_node_first_ancestor_of_type_and_its_child(
+    node: nodes.NodeNG, ancestor_type: Union[Type[T_Node], Tuple[Type[T_Node], ...]]
+) -> Union[Tuple[None, None], Tuple[T_Node, nodes.NodeNG]]:
+    """Modified version of get_node_first_ancestor_of_type to also return the
+    descendant visited directly before reaching the sought ancestor
+
+    Useful for extracting whether a statement is guarded by a try, except, or finally
+    when searching for a TryFinally ancestor.
+    """
+    child = node
+    for ancestor in node.node_ancestors():
+        if isinstance(ancestor, ancestor_type):
+            return (ancestor, child)
+        child = ancestor
+    return None, None
+
+
+def in_type_checking_block(node: nodes.NodeNG) -> bool:
+    """Check if a node is guarded by a TYPE_CHECKS guard."""
+    return any(
+        isinstance(ancestor, nodes.If)
+        and ancestor.test.as_string() in TYPING_TYPE_CHECKS_GUARDS
+        for ancestor in node.node_ancestors()
     )
