@@ -1,22 +1,21 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
 import optparse  # pylint: disable=deprecated-module
 import sys
 import time
 
 
-# pylint: disable=abstract-method; by design?
 class _ManHelpFormatter(optparse.HelpFormatter):
     def __init__(
         self, indent_increment=0, max_help_position=24, width=79, short_first=0
     ):
-        optparse.HelpFormatter.__init__(
-            self, indent_increment, max_help_position, width, short_first
-        )
+        super().__init__(indent_increment, max_help_position, width, short_first)
+        self.output_level: int
 
     def format_heading(self, heading):
-        return ".SH %s\n" % heading.upper()
+        return f".SH {heading.upper()}\n"
 
     def format_description(self, description):
         return description
@@ -54,7 +53,10 @@ class _ManHelpFormatter(optparse.HelpFormatter):
 
     @staticmethod
     def format_title(pgm, section):
-        date = "%d-%02d-%02d" % time.localtime()[:3]
+        date = (
+            "%d-%02d-%02d"  # pylint: disable=consider-using-f-string
+            % time.localtime()[:3]
+        )
         return f'.TH {pgm} {section} "{date}" {pgm}'
 
     @staticmethod
@@ -105,6 +107,10 @@ Please report bugs on the project\'s mailing list:
 """
         return tail
 
+    def format_usage(self, usage):
+        """Taken from optparse.IndentedHelpFormatter."""
+        return f"Usage: {usage}\n"
+
 
 def _generate_manpage(optparser, pkginfo, section=1, stream=sys.stdout, level=0):
     formatter = _ManHelpFormatter()
