@@ -1,9 +1,12 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
 import copy
 import optparse  # pylint: disable=deprecated-module
+import pathlib
 import re
+from typing import List, Pattern, Union
 
 from pylint import utils
 
@@ -25,6 +28,23 @@ def _regexp_csv_validator(_, name, value):
     return [_regexp_validator(_, name, val) for val in _csv_validator(_, name, value)]
 
 
+def _regexp_paths_csv_validator(
+    _, name: str, value: Union[str, List[Pattern[str]]]
+) -> List[Pattern[str]]:
+    if isinstance(value, list):
+        return value
+    patterns = []
+    for val in _csv_validator(_, name, value):
+        patterns.append(
+            re.compile(
+                str(pathlib.PureWindowsPath(val)).replace("\\", "\\\\")
+                + "|"
+                + pathlib.PureWindowsPath(val).as_posix()
+            )
+        )
+    return patterns
+
+
 def _choice_validator(choices, name, value):
     if value not in choices:
         msg = "option %s: invalid value: %r, should be in %s"
@@ -35,9 +55,11 @@ def _choice_validator(choices, name, value):
 def _yn_validator(opt, _, value):
     if isinstance(value, int):
         return bool(value)
-    if value in ("y", "yes", "true"):
+    if isinstance(value, str):
+        value = value.lower()
+    if value in {"y", "yes", "true"}:
         return True
-    if value in ("n", "no", "false"):
+    if value in {"n", "no", "false"}:
         return False
     msg = "option %s: invalid yn value %r, should be in (y, yes, true, n, no, false)"
     raise optparse.OptionValueError(msg % (opt, value))
@@ -68,7 +90,9 @@ def _py_version_validator(_, name, value):
         try:
             value = tuple(int(val) for val in value.split("."))
         except (ValueError, AttributeError):
-            raise optparse.OptionValueError(f"Invalid format for {name}") from None
+            raise optparse.OptionValueError(
+                f"Invalid format for {name}, should be version string. E.g., '3.8'"
+            ) from None
     return value
 
 
@@ -76,8 +100,9 @@ VALIDATORS = {
     "string": utils._unquote,
     "int": int,
     "float": float,
-    "regexp": re.compile,
+    "regexp": lambda pattern: re.compile(pattern or ""),
     "regexp_csv": _regexp_csv_validator,
+    "regexp_paths_csv": _regexp_paths_csv_validator,
     "csv": _csv_validator,
     "yn": _yn_validator,
     "choice": lambda opt, name, value: _choice_validator(opt["choices"], name, value),
@@ -91,7 +116,7 @@ VALIDATORS = {
 
 def _call_validator(opttype, optdict, option, value):
     if opttype not in VALIDATORS:
-        raise Exception('Unsupported type "%s"' % opttype)
+        raise Exception(f'Unsupported type "{opttype}"')
     try:
         return VALIDATORS[opttype](optdict, option, value)
     except TypeError:
@@ -104,7 +129,7 @@ def _call_validator(opttype, optdict, option, value):
 
 
 def _validate(value, optdict, name=""):
-    """return a validated value for an option according to its type
+    """Return a validated value for an option according to its type.
 
     optional argument name is only used for error message formatting
     """
@@ -120,6 +145,7 @@ class Option(optparse.Option):
     TYPES = optparse.Option.TYPES + (
         "regexp",
         "regexp_csv",
+        "regexp_paths_csv",
         "csv",
         "yn",
         "multiple_choice",
@@ -130,6 +156,7 @@ class Option(optparse.Option):
     TYPE_CHECKER = copy.copy(optparse.Option.TYPE_CHECKER)
     TYPE_CHECKER["regexp"] = _regexp_validator
     TYPE_CHECKER["regexp_csv"] = _regexp_csv_validator
+    TYPE_CHECKER["regexp_paths_csv"] = _regexp_paths_csv_validator
     TYPE_CHECKER["csv"] = _csv_validator
     TYPE_CHECKER["yn"] = _yn_validator
     TYPE_CHECKER["multiple_choice"] = _multiple_choices_validating_option
@@ -137,29 +164,29 @@ class Option(optparse.Option):
     TYPE_CHECKER["py_version"] = _py_version_validator
 
     def __init__(self, *opts, **attrs):
-        optparse.Option.__init__(self, *opts, **attrs)
+        super().__init__(*opts, **attrs)
         if hasattr(self, "hide") and self.hide:
             self.help = optparse.SUPPRESS_HELP
 
     def _check_choice(self):
-        if self.type in ("choice", "multiple_choice"):
+        if self.type in {"choice", "multiple_choice"}:
             if self.choices is None:
                 raise optparse.OptionError(
                     "must supply a list of choices for type 'choice'", self
                 )
             if not isinstance(self.choices, (tuple, list)):
                 raise optparse.OptionError(
+                    # pylint: disable-next=consider-using-f-string
                     "choices must be a list of strings ('%s' supplied)"
                     % str(type(self.choices)).split("'")[1],
                     self,
                 )
         elif self.choices is not None:
             raise optparse.OptionError(
-                "must not supply choices for type %r" % self.type, self
+                f"must not supply choices for type {self.type!r}", self
             )
 
-    # pylint: disable=unsupported-assignment-operation
-    optparse.Option.CHECK_METHODS[2] = _check_choice  # type: ignore
+    optparse.Option.CHECK_METHODS[2] = _check_choice  # type: ignore[index]
 
     def process(self, opt, value, values, parser):
         # First, convert the value(s) to the right type.  Howl if any
