@@ -1,26 +1,32 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
-
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
 import collections
 import configparser
 import contextlib
 import copy
 import functools
-import optparse
+import optparse  # pylint: disable=deprecated-module
 import os
 import sys
-
-import toml
+from pathlib import Path
+from types import ModuleType
+from typing import Dict, List, Optional, TextIO, Tuple, Union
 
 from pylint import utils
 from pylint.config.man_help_formatter import _ManHelpFormatter
 from pylint.config.option import Option
 from pylint.config.option_parser import OptionParser
 
+if sys.version_info >= (3, 11):
+    import tomllib
+else:
+    import tomli as tomllib
+
 
 def _expand_default(self, option):
-    """Patch OptionParser.expand_default with custom behaviour
+    """Patch OptionParser.expand_default with custom behaviour.
 
     This will handle defaults to avoid overriding values in the
     configuration file.
@@ -44,6 +50,7 @@ def _expand_default(self, option):
 
 @contextlib.contextmanager
 def _patch_optparse():
+    # pylint: disable = redefined-variable-type
     orig_default = optparse.HelpFormatter
     try:
         optparse.HelpFormatter.expand_default = _expand_default
@@ -53,7 +60,7 @@ def _patch_optparse():
 
 
 class OptionsManagerMixIn:
-    """Handle configuration from both a configuration file and command line options"""
+    """Handle configuration from both a configuration file and command line options."""
 
     def __init__(self, usage, config_file=None):
         self.config_file = config_file
@@ -79,10 +86,10 @@ class OptionsManagerMixIn:
         self._optik_option_attrs = set(self.cmdline_parser.option_class.ATTRS)
 
     def register_options_provider(self, provider, own_group=True):
-        """register an options provider"""
+        """Register an options provider."""
         assert provider.priority <= 0, "provider's priority can't be >= 0"
-        for i in range(len(self.options_providers)):
-            if provider.priority > self.options_providers[i].priority:
+        for i, options_provider in enumerate(self.options_providers):
+            if provider.priority > options_provider.priority:
                 self.options_providers.insert(i, provider)
                 break
         else:
@@ -138,7 +145,7 @@ class OptionsManagerMixIn:
         self._maxlevel = max(self._maxlevel, option.level or 0)
 
     def optik_option(self, provider, opt, optdict):
-        """get our personal option definition and return a suitable form for
+        """Get our personal option definition and return a suitable form for
         use with optik/optparse
         """
         optdict = copy.copy(optdict)
@@ -169,7 +176,7 @@ class OptionsManagerMixIn:
         return args, optdict
 
     def cb_set_provider_option(self, option, opt, value, parser):
-        """optik callback for option setting"""
+        """Optik callback for option setting."""
         if opt.startswith("--"):
             # remove -- on long option
             opt = opt[2:]
@@ -182,14 +189,16 @@ class OptionsManagerMixIn:
         self.global_set_option(opt, value)
 
     def global_set_option(self, opt, value):
-        """set option on the correct option provider"""
+        """Set option on the correct option provider."""
         self._all_options[opt].set_option(opt, value)
 
-    def generate_config(self, stream=None, skipsections=()):
-        """write a configuration file according to the current configuration
+    def generate_config(
+        self, stream: Optional[TextIO] = None, skipsections: Tuple[str, ...] = ()
+    ) -> None:
+        """Write a configuration file according to the current configuration
         into the given stream or stdout
         """
-        options_by_section = {}
+        options_by_section: Dict[str, List[Tuple]] = {}
         sections = []
         for provider in self.options_providers:
             for section, options in provider.options_by_section():
@@ -206,8 +215,8 @@ class OptionsManagerMixIn:
                     continue
                 if section not in sections:
                     sections.append(section)
-                alloptions = options_by_section.setdefault(section, [])
-                alloptions += options
+                all_options = options_by_section.setdefault(section, [])
+                all_options += options
         stream = stream or sys.stdout
         printed = False
         for section in sections:
@@ -218,7 +227,9 @@ class OptionsManagerMixIn:
             )
             printed = True
 
-    def generate_manpage(self, pkginfo, section=1, stream=sys.stdout):
+    def generate_manpage(
+        self, pkginfo: ModuleType, section: int = 1, stream: TextIO = sys.stdout
+    ) -> None:
         with _patch_optparse():
             formatter = _ManHelpFormatter()
             formatter.output_level = self._maxlevel
@@ -231,21 +242,20 @@ class OptionsManagerMixIn:
             print(formatter.format_tail(pkginfo), file=stream)
 
     def load_provider_defaults(self):
-        """initialize configuration using default values"""
+        """Initialize configuration using default values."""
         for provider in self.options_providers:
             provider.load_defaults()
 
     def read_config_file(self, config_file=None, verbose=None):
         """Read the configuration file but do not load it (i.e. dispatching
-        values to each options provider)
+        values to each option's provider)
         """
-        help_level = 1
-        while help_level <= self._maxlevel:
+        for help_level in range(1, self._maxlevel + 1):
             opt = "-".join(["long"] * help_level) + "-help"
             if opt in self._all_options:
                 break  # already processed
             help_function = functools.partial(self.helpfunc, level=help_level)
-            help_msg = "%s verbose help." % " ".join(["more"] * help_level)
+            help_msg = f"{' '.join(['more'] * help_level)} verbose help."
             optdict = {
                 "action": "callback",
                 "callback": help_function,
@@ -254,44 +264,28 @@ class OptionsManagerMixIn:
             provider = self.options_providers[0]
             self.add_optik_option(provider, self.cmdline_parser, opt, optdict)
             provider.options += ((opt, optdict),)
-            help_level += 1
+
         if config_file is None:
             config_file = self.config_file
         if config_file is not None:
-            config_file = os.path.expanduser(config_file)
+            config_file = os.path.expandvars(os.path.expanduser(config_file))
             if not os.path.exists(config_file):
                 raise OSError(f"The config file {config_file} doesn't exist!")
 
         use_config_file = config_file and os.path.exists(config_file)
-        if use_config_file:  # pylint: disable=too-many-nested-blocks
+        if use_config_file:
+            self.set_current_module(config_file)
             parser = self.cfgfile_parser
-
             if config_file.endswith(".toml"):
-                with open(config_file) as fp:
-                    content = toml.load(fp)
-
                 try:
-                    sections_values = content["tool"]["pylint"]
-                except KeyError:
-                    pass
-                else:
-                    for section, values in sections_values.items():
-                        # TOML has rich types, convert values to
-                        # strings as ConfigParser expects.
-                        for option, value in values.items():
-                            if isinstance(value, bool):
-                                values[option] = "yes" if value else "no"
-                            elif isinstance(value, int):
-                                values[option] = str(value)
-                            elif isinstance(value, list):
-                                values[option] = ",".join(value)
-                        parser._sections[section.upper()] = values
+                    self._parse_toml(config_file, parser)
+                except tomllib.TOMLDecodeError as e:
+                    self.add_message("config-parse-error", line=0, args=str(e))
             else:
                 # Use this encoding in order to strip the BOM marker, if any.
                 with open(config_file, encoding="utf_8_sig") as fp:
                     parser.read_file(fp)
-
-                # normalize sections'title
+                # normalize each section's title
                 for sect, values in list(parser._sections.items()):
                     if sect.startswith("pylint."):
                         sect = sect[len("pylint.") :]
@@ -305,9 +299,44 @@ class OptionsManagerMixIn:
             msg = "No config file found, using default configuration"
         print(msg, file=sys.stderr)
 
+    def _parse_toml(
+        self, config_file: Union[Path, str], parser: configparser.ConfigParser
+    ) -> None:
+        """Parse and handle errors of a toml configuration file."""
+        with open(config_file, mode="rb") as fp:
+            content = tomllib.load(fp)
+        try:
+            sections_values = content["tool"]["pylint"]
+        except KeyError:
+            return
+        for section, values in sections_values.items():
+            section_name = section.upper()
+            # TOML has rich types, convert values to
+            # strings as ConfigParser expects.
+            if not isinstance(values, dict):
+                # This class is a mixin: add_message comes from the `PyLinter` class
+                self.add_message(  # type: ignore[attr-defined]
+                    "bad-configuration-section", line=0, args=(section, values)
+                )
+                continue
+            for option, value in values.items():
+                if isinstance(value, bool):
+                    values[option] = "yes" if value else "no"
+                elif isinstance(value, list):
+                    values[option] = ",".join(value)
+                else:
+                    values[option] = str(value)
+            for option, value in values.items():
+                try:
+                    parser.set(section_name, option, value=value)
+                except configparser.NoSectionError:
+                    parser.add_section(section_name)
+                    parser.set(section_name, option, value=value)
+
     def load_config_file(self):
         """Dispatch values previously read from a configuration file to each
-        options provider)"""
+        option's provider
+        """
         parser = self.cfgfile_parser
         for section in parser.sections():
             for option, value in parser.items(section):
@@ -317,7 +346,7 @@ class OptionsManagerMixIn:
                     continue
 
     def load_configuration(self, **kwargs):
-        """override configuration according to given parameters"""
+        """Override configuration according to given parameters."""
         return self.load_configuration_from_config(kwargs)
 
     def load_configuration_from_config(self, config):
@@ -326,16 +355,13 @@ class OptionsManagerMixIn:
             provider = self._all_options[opt]
             provider.set_option(opt, opt_value)
 
-    def load_command_line_configuration(self, args=None):
-        """Override configuration according to command line parameters
+    def load_command_line_configuration(self, args=None) -> List[str]:
+        """Override configuration according to command line parameters.
 
         return additional arguments
         """
         with _patch_optparse():
-            if args is None:
-                args = sys.argv[1:]
-            else:
-                args = list(args)
+            args = sys.argv[1:] if args is None else list(args)
             (options, args) = self.cmdline_parser.parse_args(args=args)
             for provider in self._nocallback_options:
                 config = provider.config
@@ -347,7 +373,7 @@ class OptionsManagerMixIn:
             return args
 
     def add_help_section(self, title, description, level=0):
-        """add a dummy option section for help purpose"""
+        """Add a dummy option section for help purpose."""
         group = optparse.OptionGroup(
             self.cmdline_parser, title=title.capitalize(), description=description
         )
@@ -356,7 +382,7 @@ class OptionsManagerMixIn:
         self.cmdline_parser.add_option_group(group)
 
     def help(self, level=0):
-        """return the usage string for available options"""
+        """Return the usage string for available options."""
         self.cmdline_parser.formatter.output_level = level
         with _patch_optparse():
             return self.cmdline_parser.format_help()
