@@ -1,19 +1,19 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
-
-import optparse
+import optparse  # pylint: disable=deprecated-module
 from typing import Any, Dict, Tuple
 
 from pylint.config.option import _validate
 
 
 class UnsupportedAction(Exception):
-    """raised by set_option when it doesn't know what to do for an action"""
+    """Raised by set_option when it doesn't know what to do for an action."""
 
 
 class OptionsProviderMixIn:
-    """Mixin to provide options to an OptionsManager"""
+    """Mixin to provide options to an OptionsManager."""
 
     # those attributes should be overridden
     priority = -1
@@ -26,7 +26,7 @@ class OptionsProviderMixIn:
         self.load_defaults()
 
     def load_defaults(self):
-        """initialize the provider using default values"""
+        """Initialize the provider using default values."""
         for opt, optdict in self.options:
             action = optdict.get("action")
             if action != "callback":
@@ -37,17 +37,17 @@ class OptionsProviderMixIn:
                 self.set_option(opt, default, action, optdict)
 
     def option_attrname(self, opt, optdict=None):
-        """get the config attribute corresponding to opt"""
+        """Get the config attribute corresponding to opt."""
         if optdict is None:
             optdict = self.get_option_def(opt)
         return optdict.get("dest", opt.replace("-", "_"))
 
     def option_value(self, opt):
-        """get the current value for the given option"""
+        """Get the current value for the given option."""
         return getattr(self.config, self.option_attrname(opt), None)
 
     def set_option(self, optname, value, action=None, optdict=None):
-        """method called to set an option (registered in the options list)"""
+        """Method called to set an option (registered in the options list)."""
         if optdict is None:
             optdict = self.get_option_def(optname)
         if value is not None:
@@ -56,7 +56,7 @@ class OptionsProviderMixIn:
             action = optdict.get("action", "store")
         if action == "store":
             setattr(self.config, self.option_attrname(optname, optdict), value)
-        elif action in ("store_true", "count"):
+        elif action in {"store_true", "count"}:
             setattr(self.config, self.option_attrname(optname, optdict), 0)
         elif action == "store_false":
             setattr(self.config, self.option_attrname(optname, optdict), 1)
@@ -80,7 +80,7 @@ class OptionsProviderMixIn:
             raise UnsupportedAction(action)
 
     def get_option_def(self, opt):
-        """return the dictionary defining an option given its name"""
+        """Return the dictionary defining an option given its name."""
         assert self.options
         for option in self.options:
             if option[0] == opt:
@@ -90,7 +90,7 @@ class OptionsProviderMixIn:
         )
 
     def options_by_section(self):
-        """return an iterator on options grouped by section
+        """Return an iterator on options grouped by section.
 
         (section, [list of (optname, optdict, optvalue)])
         """
