@@ -1,18 +1,17 @@
-# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
-# Copyright (c) 2021 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-# Copyright (c) 2021 Andreas Finkler <andi.finkler@gmail.com>
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
+
+from typing import TYPE_CHECKING
 
 from astroid import nodes
 
 from pylint.checkers import BaseChecker
 from pylint.checkers.utils import check_messages
 from pylint.interfaces import IAstroidChecker
-from pylint.lint import PyLinter
+
+if TYPE_CHECKING:
+    from pylint.lint import PyLinter
 
 
 class ConfusingConsecutiveElifChecker(BaseChecker):
@@ -50,10 +49,5 @@ class ConfusingConsecutiveElifChecker(BaseChecker):
         return False
 
 
-def register(linter: PyLinter):
-    """This required method auto registers the checker.
-
-    :param linter: The linter to register the checker to.
-    :type linter: pylint.lint.PyLinter
-    """
+def register(linter: "PyLinter") -> None:
     linter.register_checker(ConfusingConsecutiveElifChecker(linter))
