@@ -1,30 +1,25 @@
-# Copyright (c) 2016 Alexander Todorov <atodorov@otb.bg>
-# Copyright (c) 2017-2018, 2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2019, 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
 """Looks for  comparisons to empty string."""
 
 import itertools
+from typing import TYPE_CHECKING, Any, Iterable
 
 from astroid import nodes
 
 from pylint import checkers, interfaces
 from pylint.checkers import utils
 
-
-def _is_constant_empty_str(node):
-    return isinstance(node, nodes.Const) and node.value == ""
+if TYPE_CHECKING:
+    from pylint.lint import PyLinter
 
 
 class CompareToEmptyStringChecker(checkers.BaseChecker):
     """Checks for comparisons to empty string.
-    Most of the times you should use the fact that empty strings are false.
+
+    Most of the time you should use the fact that empty strings are false.
     An exception to this rule is when an empty string value is allowed in the program
     and has a different meaning than None!
     """
@@ -45,7 +40,7 @@ class CompareToEmptyStringChecker(checkers.BaseChecker):
     options = ()
 
     @utils.check_messages("compare-to-empty-string")
-    def visit_compare(self, node):
+    def visit_compare(self, node: nodes.Compare) -> None:
         _operators = ["!=", "==", "is not", "is"]
         # note: astroid.Compare has the left most operand in node.left
         # while the rest are a list of tuples in node.ops
@@ -53,7 +48,8 @@ class CompareToEmptyStringChecker(checkers.BaseChecker):
         # here we squash everything into `ops` to make it easier for processing later
         ops = [("", node.left)]
         ops.extend(node.ops)
-        ops = list(itertools.chain(*ops))
+        iter_ops: Iterable[Any] = iter(ops)
+        ops = list(itertools.chain(*iter_ops))
 
         for ops_idx in range(len(ops) - 2):
             op_1 = ops[ops_idx]
@@ -62,16 +58,15 @@ class CompareToEmptyStringChecker(checkers.BaseChecker):
             error_detected = False
 
             # x ?? ""
-            if _is_constant_empty_str(op_1) and op_2 in _operators:
+            if utils.is_empty_str_literal(op_1) and op_2 in _operators:
                 error_detected = True
             # '' ?? X
-            elif op_2 in _operators and _is_constant_empty_str(op_3):
+            elif op_2 in _operators and utils.is_empty_str_literal(op_3):
                 error_detected = True
 
             if error_detected:
                 self.add_message("compare-to-empty-string", node=node)
 
 
-def register(linter):
-    """Required method to auto register this checker."""
+def register(linter: "PyLinter") -> None:
     linter.register_checker(CompareToEmptyStringChecker(linter))
