@@ -1,8 +1,19 @@
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
+
 """Check for use of while loops."""
+from typing import TYPE_CHECKING
+
+from astroid import nodes
+
 from pylint.checkers import BaseChecker
 from pylint.checkers.utils import check_messages
 from pylint.interfaces import IAstroidChecker
 
+if TYPE_CHECKING:
+    from pylint.lint import PyLinter
+
 
 class WhileChecker(BaseChecker):
 
@@ -17,14 +28,9 @@ class WhileChecker(BaseChecker):
     }
 
     @check_messages("while-used")
-    def visit_while(self, node):
+    def visit_while(self, node: nodes.While) -> None:
         self.add_message("while-used", node=node)
 
 
-def register(linter):
-    """Required method to auto register this checker.
-
-    :param linter: Main interface object for Pylint plugins
-    :type linter: Pylint object
-    """
+def register(linter: "PyLinter") -> None:
     linter.register_checker(WhileChecker(linter))
