@@ -1,12 +1,19 @@
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
+
 import os
 import sys
+from typing import List, Pattern, Tuple
 
 from astroid import modutils
 
+from pylint.typing import ErrorDescriptionDict, ModuleDescriptionDict
+
 
 def _modpath_from_file(filename, is_namespace, path=None):
-    def _is_package_cb(path, parts):
-        return modutils.check_modpath_has_init(path, parts) or is_namespace
+    def _is_package_cb(inner_path, parts):
+        return modutils.check_modpath_has_init(inner_path, parts) or is_namespace
 
     return modutils.modpath_from_file_with_callback(
         filename, path=path, is_package_cb=_is_package_cb
@@ -15,7 +22,10 @@ def _modpath_from_file(filename, is_namespace, path=None):
 
 def get_python_path(filepath: str) -> str:
     """TODO This get the python path with the (bad) assumption that there is always
-    an __init__.py. This is not true since python 3.3 and is causing problem."""
+    an __init__.py
+
+    This is not true since python 3.3 and is causing problem.
+    """
     dirname = os.path.realpath(os.path.expanduser(filepath))
     if not os.path.isdir(dirname):
         dirname = os.path.dirname(dirname)
@@ -28,32 +38,30 @@ def get_python_path(filepath: str) -> str:
             return os.getcwd()
 
 
-def _basename_in_ignore_list_re(base_name, ignore_list_re):
-    """Determines if the basename is matched in a regex ignorelist
-
-    :param str base_name: The basename of the file
-    :param list ignore_list_re: A collection of regex patterns to match against.
-        Successful matches are ignored.
-
-    :returns: `True` if the basename is ignored, `False` otherwise.
-    :rtype: bool
-    """
-    for file_pattern in ignore_list_re:
-        if file_pattern.match(base_name):
-            return True
-    return False
+def _is_in_ignore_list_re(element: str, ignore_list_re: List[Pattern]) -> bool:
+    """Determines if the element is matched in a regex ignore-list."""
+    return any(file_pattern.match(element) for file_pattern in ignore_list_re)
 
 
-def expand_modules(files_or_modules, ignore_list, ignore_list_re):
+def expand_modules(
+    files_or_modules: List[str],
+    ignore_list: List[str],
+    ignore_list_re: List[Pattern],
+    ignore_list_paths_re: List[Pattern[str]],
+) -> Tuple[List[ModuleDescriptionDict], List[ErrorDescriptionDict]]:
     """Take a list of files/modules/packages and return the list of tuple
-    (file, module name) which have to be actually checked."""
-    result = []
-    errors = []
+    (file, module name) which have to be actually checked
+    """
+    result: List[ModuleDescriptionDict] = []
+    errors: List[ErrorDescriptionDict] = []
     path = sys.path.copy()
+
     for something in files_or_modules:
         basename = os.path.basename(something)
-        if basename in ignore_list or _basename_in_ignore_list_re(
-            basename, ignore_list_re
+        if (
+            basename in ignore_list
+            or _is_in_ignore_list_re(os.path.basename(something), ignore_list_re)
+            or _is_in_ignore_list_re(something, ignore_list_paths_re)
         ):
             continue
         module_path = get_python_path(something)
@@ -117,10 +125,11 @@ def expand_modules(files_or_modules, ignore_list, ignore_list_re):
             ):
                 if filepath == subfilepath:
                     continue
-                if _basename_in_ignore_list_re(
+                if _is_in_ignore_list_re(
                     os.path.basename(subfilepath), ignore_list_re
-                ):
+                ) or _is_in_ignore_list_re(subfilepath, ignore_list_paths_re):
                     continue
+
                 modpath = _modpath_from_file(
                     subfilepath, is_namespace, path=additional_search_path
                 )
