@@ -1,13 +1,18 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
 import collections
-from typing import Dict, List, Tuple, ValuesView
+import functools
+from typing import TYPE_CHECKING, Dict, List, Tuple, ValuesView
 
 from pylint.exceptions import UnknownMessageError
 from pylint.message.message_definition import MessageDefinition
 from pylint.message.message_id_store import MessageIdStore
 
+if TYPE_CHECKING:
+    from pylint.checkers import BaseChecker
+
 
 class MessageDefinitionStore:
 
@@ -15,7 +20,7 @@ class MessageDefinitionStore:
     no particular state during analysis.
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.message_id_store: MessageIdStore = MessageIdStore()
         # Primary registry for all active messages definitions.
         # It contains the 1:1 mapping from msgid to MessageDefinition.
@@ -29,7 +34,7 @@ class MessageDefinitionStore:
         """The list of all active messages."""
         return self._messages_definitions.values()
 
-    def register_messages_from_checker(self, checker) -> None:
+    def register_messages_from_checker(self, checker: "BaseChecker") -> None:
         """Register all messages definitions from a checker."""
         checker.check_consistency()
         for message in checker.messages:
@@ -43,8 +48,18 @@ class MessageDefinitionStore:
         self._messages_definitions[message.msgid] = message
         self._msgs_by_category[message.msgid[0]].append(message.msgid)
 
+    # Since MessageDefinitionStore is only initialized once
+    # and the arguments are relatively small in size we do not run the
+    # risk of creating a large memory leak.
+    # See discussion in: https://github.com/PyCQA/pylint/pull/5673
+    @functools.lru_cache(maxsize=None)  # pylint: disable=cache-max-size-none
     def get_message_definitions(self, msgid_or_symbol: str) -> List[MessageDefinition]:
-        """Returns the Message definition for either a numeric or symbolic id."""
+        """Returns the Message definition for either a numeric or symbolic id.
+
+        The cache has no limit as its size will likely stay minimal. For each message we store
+        about 1000 characters, so even if we would have 1000 messages the cache would only
+        take up ~= 1 Mb.
+        """
         return [
             self._messages_definitions[m]
             for m in self.message_id_store.get_active_msgids(msgid_or_symbol)
@@ -58,7 +73,7 @@ class MessageDefinitionStore:
         return repr([md.symbol for md in message_definitions])
 
     def help_message(self, msgids_or_symbols: List[str]) -> None:
-        """Display help messages for the given message identifiers"""
+        """Display help messages for the given message identifiers."""
         for msgids_or_symbol in msgids_or_symbols:
             try:
                 for message_definition in self.get_message_definitions(
@@ -85,7 +100,7 @@ class MessageDefinitionStore:
     def find_emittable_messages(
         self,
     ) -> Tuple[List[MessageDefinition], List[MessageDefinition]]:
-        """Finds all emittable and non-emittable messages"""
+        """Finds all emittable and non-emittable messages."""
         messages = sorted(self._messages_definitions.values(), key=lambda m: m.msgid)
         emittable = []
         non_emittable = []
