@@ -1,53 +1,40 @@
-# Copyright (c) 2006, 2008-2010, 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2014 Brett Cannon <brett@python.org>
-# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
-# Copyright (c) 2015-2018, 2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
-# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
-# Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
-# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
-# Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
-# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
-# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
-"""handle diagram generation options for class diagram or default diagrams
-"""
+"""Handle diagram generation options for class diagram or default diagrams."""
+
+from typing import Any, Optional
 
 import astroid
+from astroid import nodes
 
 from pylint.pyreverse.diagrams import ClassDiagram, PackageDiagram
+from pylint.pyreverse.inspector import Project
 from pylint.pyreverse.utils import LocalsVisitor
 
-BUILTINS_NAME = "builtins"
-
 # diagram generators ##########################################################
 
 
 class DiaDefGenerator:
-    """handle diagram generation options"""
+    """Handle diagram generation options."""
 
     def __init__(self, linker, handler):
-        """common Diagram Handler initialization"""
+        """Common Diagram Handler initialization."""
         self.config = handler.config
         self._set_default_options()
         self.linker = linker
         self.classdiagram = None  # defined by subclasses
 
     def get_title(self, node):
-        """get title for objects"""
+        """Get title for objects."""
         title = node.name
         if self.module_names:
             title = f"{node.root().name}.{title}"
         return title
 
     def _set_option(self, option):
-        """activate some options if not explicitly deactivated"""
+        """Activate some options if not explicitly deactivated."""
         # if we have a class diagram, we want more information by default;
         # so if the option is None, we return True
         if option is None:
@@ -55,7 +42,7 @@ class DiaDefGenerator:
         return option
 
     def _set_default_options(self):
-        """set different default options with _default dictionary"""
+        """Set different default options with _default dictionary."""
         self.module_names = self._set_option(self.config.module_names)
         all_ancestors = self._set_option(self.config.all_ancestors)
         all_associated = self._set_option(self.config.all_associated)
@@ -71,22 +58,22 @@ class DiaDefGenerator:
         self.anc_level, self.association_level = anc_level, association_level
 
     def _get_levels(self):
-        """help function for search levels"""
+        """Help function for search levels."""
         return self.anc_level, self.association_level
 
     def show_node(self, node):
-        """true if builtins and not show_builtins"""
+        """True if builtins and not show_builtins."""
         if self.config.show_builtin:
             return True
-        return node.root().name != BUILTINS_NAME
+        return node.root().name != "builtins"
 
     def add_class(self, node):
-        """visit one class and add it to diagram"""
+        """Visit one class and add it to diagram."""
         self.linker.visit(node)
         self.classdiagram.add_object(self.get_title(node), node)
 
     def get_ancestors(self, node, level):
-        """return ancestor nodes of a class node"""
+        """Return ancestor nodes of a class node."""
         if level == 0:
             return
         for ancestor in node.ancestors(recurs=False):
@@ -95,7 +82,7 @@ class DiaDefGenerator:
             yield ancestor
 
     def get_associated(self, klass_node, level):
-        """return associated nodes of a class node"""
+        """Return associated nodes of a class node."""
         if level == 0:
             return
         for association_nodes in list(klass_node.instance_attrs_type.values()) + list(
@@ -104,12 +91,12 @@ class DiaDefGenerator:
             for node in association_nodes:
                 if isinstance(node, astroid.Instance):
                     node = node._proxied
-                if not (isinstance(node, astroid.ClassDef) and self.show_node(node)):
+                if not (isinstance(node, nodes.ClassDef) and self.show_node(node)):
                     continue
                 yield node
 
     def extract_classes(self, klass_node, anc_level, association_level):
-        """extract recursively classes related to klass_node"""
+        """Extract recursively classes related to klass_node."""
         if self.classdiagram.has_node(klass_node) or not self.show_node(klass_node):
             return
         self.add_class(klass_node)
@@ -122,7 +109,7 @@ class DiaDefGenerator:
 
 
 class DefaultDiadefGenerator(LocalsVisitor, DiaDefGenerator):
-    """generate minimum diagram definition for the project :
+    """Generate minimum diagram definition for the project :
 
     * a package diagram including project's modules
     * a class diagram including project's classes
@@ -132,20 +119,22 @@ class DefaultDiadefGenerator(LocalsVisitor, DiaDefGenerator):
         DiaDefGenerator.__init__(self, linker, handler)
         LocalsVisitor.__init__(self)
 
-    def visit_project(self, node):
-        """visit a pyreverse.utils.Project node
+    def visit_project(self, node: Project) -> None:
+        """Visit a pyreverse.utils.Project node.
 
         create a diagram definition for packages
         """
         mode = self.config.mode
         if len(node.modules) > 1:
-            self.pkgdiagram = PackageDiagram("packages %s" % node.name, mode)
+            self.pkgdiagram: Optional[PackageDiagram] = PackageDiagram(
+                f"packages {node.name}", mode
+            )
         else:
             self.pkgdiagram = None
-        self.classdiagram = ClassDiagram("classes %s" % node.name, mode)
+        self.classdiagram = ClassDiagram(f"classes {node.name}", mode)
 
-    def leave_project(self, node):  # pylint: disable=unused-argument
-        """leave the pyreverse.utils.Project node
+    def leave_project(self, _: Project) -> Any:
+        """Leave the pyreverse.utils.Project node.
 
         return the generated diagram definition
         """
@@ -153,8 +142,8 @@ class DefaultDiadefGenerator(LocalsVisitor, DiaDefGenerator):
             return self.pkgdiagram, self.classdiagram
         return (self.classdiagram,)
 
-    def visit_module(self, node):
-        """visit an astroid.Module node
+    def visit_module(self, node: nodes.Module) -> None:
+        """Visit an astroid.Module node.
 
         add this class to the package diagram definition
         """
@@ -162,30 +151,27 @@ class DefaultDiadefGenerator(LocalsVisitor, DiaDefGenerator):
             self.linker.visit(node)
             self.pkgdiagram.add_object(node.name, node)
 
-    def visit_classdef(self, node):
-        """visit an astroid.Class node
+    def visit_classdef(self, node: nodes.ClassDef) -> None:
+        """Visit an astroid.Class node.
 
         add this class to the class diagram definition
         """
         anc_level, association_level = self._get_levels()
         self.extract_classes(node, anc_level, association_level)
 
-    def visit_importfrom(self, node):
-        """visit astroid.ImportFrom  and catch modules for package diagram"""
+    def visit_importfrom(self, node: nodes.ImportFrom) -> None:
+        """Visit astroid.ImportFrom  and catch modules for package diagram."""
         if self.pkgdiagram:
             self.pkgdiagram.add_from_depend(node, node.modname)
 
 
 class ClassDiadefGenerator(DiaDefGenerator):
-    """generate a class diagram definition including all classes related to a
+    """Generate a class diagram definition including all classes related to a
     given class
     """
 
-    def __init__(self, linker, handler):
-        DiaDefGenerator.__init__(self, linker, handler)
-
     def class_diagram(self, project, klass):
-        """return a class diagram definition for the given klass and its
+        """Return a class diagram definition for the given klass and its
         related klasses
         """
 
@@ -207,16 +193,13 @@ class ClassDiadefGenerator(DiaDefGenerator):
 
 
 class DiadefsHandler:
-    """handle diagram definitions :
-
-    get it from user (i.e. xml files) or generate them
-    """
+    """Get diagram definitions from user (i.e. xml files) or generate them."""
 
     def __init__(self, config):
         self.config = config
 
     def get_diadefs(self, project, linker):
-        """Get the diagrams configuration data
+        """Get the diagram's configuration data.
 
         :param project:The pyreverse project
         :type project: pyreverse.utils.Project
