@@ -1,27 +1,12 @@
-# Copyright (c) 2006, 2008, 2010, 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2014 Brett Cannon <brett@python.org>
-# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
-# Copyright (c) 2015-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
-# Copyright (c) 2017, 2020 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
-# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
-# Copyright (c) 2020-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2020 yeting li <liyt@ios.ac.cn>
-# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
-# Copyright (c) 2020 bernie gray <bfgray3@users.noreply.github.com>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-# Copyright (c) 2021 Mark Byrne <31762852+mbyrnepr2@users.noreply.github.com>
-# Copyright (c) 2021 Andreas Finkler <andi.finkler@gmail.com>
-# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
-"""Generic classes/functions for pyreverse core/extensions. """
+"""Generic classes/functions for pyreverse core/extensions."""
 import os
 import re
 import shutil
+import subprocess
 import sys
 from typing import Optional, Union
 
@@ -46,7 +31,7 @@ def get_default_options():
 
 
 def insert_default_options():
-    """insert default options to sys.argv"""
+    """Insert default options to sys.argv."""
     options = get_default_options()
     options.reverse()
     for arg in options:
@@ -60,7 +45,7 @@ PROTECTED = re.compile(r"^_\w*$")
 
 
 def get_visibility(name):
-    """return the visibility from a name: public, protected, private or special"""
+    """Return the visibility from a name: public, protected, private or special."""
     if SPECIAL.match(name):
         visibility = "special"
     elif PRIVATE.match(name):
@@ -78,14 +63,14 @@ FINAL = re.compile(r"^[^\W\da-z]*$")
 
 
 def is_abstract(node):
-    """return true if the given class node correspond to an abstract class
+    """Return true if the given class node correspond to an abstract class
     definition
     """
     return ABSTRACT.match(node.name)
 
 
 def is_final(node):
-    """return true if the given class/function node correspond to final
+    """Return true if the given class/function node correspond to final
     definition
     """
     return FINAL.match(node.name)
@@ -122,26 +107,26 @@ VIS_MOD = {
 
 
 class FilterMixIn:
-    """filter nodes according to a mode and nodes' visibility"""
+    """Filter nodes according to a mode and nodes' visibility."""
 
     def __init__(self, mode):
-        "init filter modes"
+        """Init filter modes."""
         __mode = 0
         for nummod in mode.split("+"):
             try:
                 __mode += MODES[nummod]
             except KeyError as ex:
-                print("Unknown filter mode %s" % ex, file=sys.stderr)
+                print(f"Unknown filter mode {ex}", file=sys.stderr)
         self.__mode = __mode
 
     def show_attr(self, node):
-        """return true if the node should be treated"""
+        """Return true if the node should be treated."""
         visibility = get_visibility(getattr(node, "name", node))
         return not self.__mode & VIS_MOD[visibility]
 
 
 class ASTWalker:
-    """a walker visiting a tree in preorder, calling on the handler:
+    """A walker visiting a tree in preorder, calling on the handler:.
 
     * visit_<class name> on entering a node, where class name is the class of
     the node in lower case
@@ -155,7 +140,7 @@ class ASTWalker:
         self._cache = {}
 
     def walk(self, node, _done=None):
-        """walk on the tree from <node>, getting callbacks from handler"""
+        """Walk on the tree from <node>, getting callbacks from handler."""
         if _done is None:
             _done = set()
         if node in _done:
@@ -169,17 +154,17 @@ class ASTWalker:
         assert node.parent is not node
 
     def get_callbacks(self, node):
-        """get callbacks from handler for the visited node"""
+        """Get callbacks from handler for the visited node."""
         klass = node.__class__
         methods = self._cache.get(klass)
         if methods is None:
             handler = self.handler
             kid = klass.__name__.lower()
             e_method = getattr(
-                handler, "visit_%s" % kid, getattr(handler, "visit_default", None)
+                handler, f"visit_{kid}", getattr(handler, "visit_default", None)
             )
             l_method = getattr(
-                handler, "leave_%s" % kid, getattr(handler, "leave_default", None)
+                handler, f"leave_{kid}", getattr(handler, "leave_default", None)
             )
             self._cache[klass] = (e_method, l_method)
         else:
@@ -187,27 +172,27 @@ class ASTWalker:
         return e_method, l_method
 
     def visit(self, node):
-        """walk on the tree from <node>, getting callbacks from handler"""
+        """Walk on the tree from <node>, getting callbacks from handler."""
         method = self.get_callbacks(node)[0]
         if method is not None:
             method(node)
 
     def leave(self, node):
-        """walk on the tree from <node>, getting callbacks from handler"""
+        """Walk on the tree from <node>, getting callbacks from handler."""
         method = self.get_callbacks(node)[1]
         if method is not None:
             method(node)
 
 
 class LocalsVisitor(ASTWalker):
-    """visit a project by traversing the locals dictionary"""
+    """Visit a project by traversing the locals dictionary."""
 
     def __init__(self):
-        ASTWalker.__init__(self, self)
+        super().__init__(self)
         self._visited = set()
 
     def visit(self, node):
-        """launch the visit starting from the given node"""
+        """Launch the visit starting from the given node."""
         if node in self._visited:
             return None
 
@@ -223,19 +208,18 @@ class LocalsVisitor(ASTWalker):
         return None
 
 
-def get_annotation_label(ann: Union[nodes.Name, nodes.Subscript]) -> str:
-    label = ""
-    if isinstance(ann, nodes.Subscript):
-        label = ann.as_string()
-    elif isinstance(ann, nodes.Name):
-        label = ann.name
-    return label
+def get_annotation_label(ann: Union[nodes.Name, nodes.NodeNG]) -> str:
+    if isinstance(ann, nodes.Name) and ann.name is not None:
+        return ann.name
+    if isinstance(ann, nodes.NodeNG):
+        return ann.as_string()
+    return ""
 
 
 def get_annotation(
     node: Union[nodes.AssignAttr, nodes.AssignName]
 ) -> Optional[Union[nodes.Name, nodes.Subscript]]:
-    """return the annotation for `node`"""
+    """Return the annotation for `node`."""
     ann = None
     if isinstance(node.parent, nodes.AnnAssign):
         ann = node.parent.annotation
@@ -269,7 +253,8 @@ def get_annotation(
 
 def infer_node(node: Union[nodes.AssignAttr, nodes.AssignName]) -> set:
     """Return a set containing the node annotation if it exists
-    otherwise return a set of the inferred types using the NodeNG.infer method"""
+    otherwise return a set of the inferred types using the NodeNG.infer method
+    """
 
     ann = get_annotation(node)
     try:
@@ -284,12 +269,38 @@ def infer_node(node: Union[nodes.AssignAttr, nodes.AssignName]) -> set:
 
 def check_graphviz_availability():
     """Check if the ``dot`` command is available on the machine.
+
     This is needed if image output is desired and ``dot`` is used to convert
-    from *.dot or *.gv into the final output format."""
+    from *.dot or *.gv into the final output format.
+    """
     if shutil.which("dot") is None:
+        print("'Graphviz' needs to be installed for your chosen output format.")
+        sys.exit(32)
+
+
+def check_if_graphviz_supports_format(output_format: str) -> None:
+    """Check if the ``dot`` command supports the requested output format.
+
+    This is needed if image output is desired and ``dot`` is used to convert
+    from *.gv into the final output format.
+    """
+    dot_output = subprocess.run(
+        ["dot", "-T?"], capture_output=True, check=False, encoding="utf-8"
+    )
+    match = re.match(
+        pattern=r".*Use one of: (?P<formats>(\S*\s?)+)",
+        string=dot_output.stderr.strip(),
+    )
+    if not match:
         print(
-            "The requested output format is currently not available.\n"
-            "Please install 'Graphviz' to have other output formats "
-            "than 'dot' or 'vcg'."
+            "Unable to determine Graphviz supported output formats. "
+            "Pyreverse will continue, but subsequent error messages "
+            "regarding the output format may come from Graphviz directly."
+        )
+        return
+    supported_formats = match.group("formats")
+    if output_format not in supported_formats.split():
+        print(
+            f"Format {output_format} is not supported by Graphviz. It supports: {supported_formats}"
         )
         sys.exit(32)
