@@ -1,25 +1,27 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
 import re
 from collections import namedtuple
-from typing import Generator, List
+from typing import Generator, List, Optional
 
 # Allow stopping after the first semicolon/hash encountered,
 # so that an option can be continued with the reasons
 # why it is active or disabled.
 OPTION_RGX = r"""
-    \s*                # Any number of whithespace
-    \#?                # One or zero hash
-    .*                 # Anything (as much as possible)
-    (\s*               # Beginning of first matched group and any number of whitespaces
-    \#                 # Beginning of comment
-    .*?                # Anything (as little as possible)
-    \bpylint:          # pylint word and column
-    \s*                # Any number of whitespaces
-    ([^;#\n]+))        # Anything except semicolon or hash or newline (it is the second matched group)
-                       # and end of the first matched group
-    [;#]{0,1}"""  # From 0 to 1 repetition of semicolon or hash
+    (?:^\s*\#.*|\s*|               # Comment line, or whitespaces,
+       \s*\#.*(?=\#.*?\bpylint:))  # or a beginning of an inline comment
+                                   # followed by "pylint:" pragma
+    (\#                            # Beginning of comment
+    .*?                            # Anything (as little as possible)
+    \bpylint:                      # pylint word and column
+    \s*                            # Any number of whitespaces
+    ([^;#\n]+))                    # Anything except semicolon or hash or
+                                   # newline (it is the second matched group)
+                                   # and end of the first matched group
+    [;#]{0,1}                      # From 0 to 1 repetition of semicolon or hash
+"""
 OPTION_PO = re.compile(OPTION_RGX, re.VERBOSE)
 
 
@@ -31,18 +33,18 @@ MESSAGE_KEYWORDS = frozenset(
     ("disable-next", "disable-msg", "enable-msg", "disable", "enable")
 )
 # sorted is necessary because sets are unordered collections and ALL_KEYWORDS
-#  string should not vary between executions
-#  reverse is necessary in order to have the longest keywords first, so that, for example,
-# 'disable' string should not be matched instead of 'disable-all'
+# string should not vary between executions
+# reverse is necessary in order to have the longest keywords first, so that, for example,
+# 'disable' string should not be matched instead of 'disable-all'
 ALL_KEYWORDS = "|".join(
     sorted(ATOMIC_KEYWORDS | MESSAGE_KEYWORDS, key=len, reverse=True)
 )
 
 
 TOKEN_SPECIFICATION = [
-    ("KEYWORD", fr"\b({ALL_KEYWORDS:s})\b"),
-    ("MESSAGE_STRING", r"[0-9A-Za-z\-\_]{2,}"),  #  Identifiers
-    ("ASSIGN", r"="),  #  Assignment operator
+    ("KEYWORD", rf"\b({ALL_KEYWORDS:s})\b"),
+    ("MESSAGE_STRING", r"[0-9A-Za-z\-\_]{2,}"),  # Identifiers
+    ("ASSIGN", r"="),  # Assignment operator
     ("MESSAGE_NUMBER", r"[CREIWF]{1}\d*"),
 ]
 
@@ -52,7 +54,7 @@ TOK_REGEX = "|".join(
 )
 
 
-def emit_pragma_representer(action, messages):
+def emit_pragma_representer(action: str, messages: List[str]) -> PragmaRepresenter:
     if not messages and action in MESSAGE_KEYWORDS:
         raise InvalidPragmaError(
             "The keyword is not followed by message identifier", action
@@ -61,13 +63,10 @@ def emit_pragma_representer(action, messages):
 
 
 class PragmaParserError(Exception):
-    """
-    A class for exceptions thrown by pragma_parser module
-    """
+    """A class for exceptions thrown by pragma_parser module."""
 
-    def __init__(self, message, token):
-        """
-        :args message: explain the reason why the exception has been thrown
+    def __init__(self, message: str, token: str) -> None:
+        """:args message: explain the reason why the exception has been thrown
         :args token: token concerned by the exception
         """
         self.message = message
@@ -76,19 +75,15 @@ class PragmaParserError(Exception):
 
 
 class UnRecognizedOptionError(PragmaParserError):
-    """
-    Thrown in case the of a valid but unrecognized option
-    """
+    """Thrown in case the of a valid but unrecognized option."""
 
 
 class InvalidPragmaError(PragmaParserError):
-    """
-    Thrown in case the pragma is invalid
-    """
+    """Thrown in case the pragma is invalid."""
 
 
 def parse_pragma(pylint_pragma: str) -> Generator[PragmaRepresenter, None, None]:
-    action = None
+    action: Optional[str] = None
     messages: List[str] = []
     assignment_required = False
     previous_token = ""
@@ -100,12 +95,12 @@ def parse_pragma(pylint_pragma: str) -> Generator[PragmaRepresenter, None, None]
         if kind == "ASSIGN":
             if not assignment_required:
                 if action:
-                    # A keyword has been found previously but doesn't support assignement
+                    # A keyword has been found previously but doesn't support assignment
                     raise UnRecognizedOptionError(
                         "The keyword doesn't support assignment", action
                     )
                 if previous_token:
-                    #  Something found previously but not a known keyword
+                    # Something found previously but not a known keyword
                     raise UnRecognizedOptionError(
                         "The keyword is unknown", previous_token
                     )
@@ -113,14 +108,16 @@ def parse_pragma(pylint_pragma: str) -> Generator[PragmaRepresenter, None, None]
                 raise InvalidPragmaError("Missing keyword before assignment", "")
             assignment_required = False
         elif assignment_required:
-            raise InvalidPragmaError("The = sign is missing after the keyword", action)
+            raise InvalidPragmaError(
+                "The = sign is missing after the keyword", action or ""
+            )
         elif kind == "KEYWORD":
             if action:
                 yield emit_pragma_representer(action, messages)
             action = value
             messages = []
             assignment_required = action in MESSAGE_KEYWORDS
-        elif kind in ("MESSAGE_STRING", "MESSAGE_NUMBER"):
+        elif kind in {"MESSAGE_STRING", "MESSAGE_NUMBER"}:
             messages.append(value)
             assignment_required = False
         else:
